# -*- coding: utf-8 -*-
"""
Test the MessagePack utility
"""

# Import Python Libs
from __future__ import absolute_import

import inspect
import os
import pprint
import struct
import sys
from io import BytesIO

import salt.utils.msgpack
from salt.ext.six.moves import range

# Import Salt Libs
from salt.utils.odict import OrderedDict

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf

try:
    import msgpack
except ImportError:
    import msgpack_pure as msgpack  # pylint: disable=import-error


# A keyword to pass to tests that use `raw`, which was added in msgpack 0.5.2
raw = {"raw": False} if msgpack.version > (0, 5, 2) else {}


@skipIf(not salt.utils.msgpack.HAS_MSGPACK, "msgpack module required for these tests")
class TestMsgpack(TestCase):
    """
    In msgpack, the following aliases exist:
      load = unpack
      loads = unpackb
      dump = pack
      dumps = packb
    The salt.utils.msgpack versions of these functions are not aliases,
    verify that they pass the same relevant tests from:
        https://github.com/msgpack/msgpack-python/blob/master/test/
    """

    test_data = [
        0,
        1,
        127,
        128,
        255,
        256,
        65535,
        65536,
        4294967295,
        4294967296,
        -1,
        -32,
        -33,
        -128,
        -129,
        -32768,
        -32769,
        -4294967296,
        -4294967297,
        1.0,
        b"",
        b"a",
        b"a" * 31,
        b"a" * 32,
        None,
        True,
        False,
        (),
        ((),),
        ((), None,),
        {None: 0},
        (1 << 23),
    ]

    def test_version(self):
        """
        Verify that the version exists and returns a value in the expected format
        """
        version = salt.utils.msgpack.version
        self.assertTrue(isinstance(version, tuple))
        self.assertGreater(version, (0, 0, 0))

    def test_Packer(self):
        data = os.urandom(1024)
        packer = salt.utils.msgpack.Packer()
        unpacker = msgpack.Unpacker(None)

        packed = packer.pack(data)
        # Sanity Check
        self.assertTrue(packed)
        self.assertNotEqual(data, packed)

        # Reverse the packing and the result should be equivalent to the original data
        unpacker.feed(packed)
        unpacked = msgpack.unpackb(packed)
        self.assertEqual(data, unpacked)

    def test_Unpacker(self):
        data = os.urandom(1024)
        packer = msgpack.Packer()
        unpacker = salt.utils.msgpack.Unpacker(None)

        packed = packer.pack(data)
        # Sanity Check
        self.assertTrue(packed)
        self.assertNotEqual(data, packed)

        # Reverse the packing and the result should be equivalent to the original data
        unpacker.feed(packed)
        unpacked = msgpack.unpackb(packed)
        self.assertEqual(data, unpacked)

    def test_array_size(self):
        sizes = [0, 5, 50, 1000]
        bio = BytesIO()
        packer = salt.utils.msgpack.Packer()
        for size in sizes:
            bio.write(packer.pack_array_header(size))
            for i in range(size):
                bio.write(packer.pack(i))

        bio.seek(0)
        unpacker = salt.utils.msgpack.Unpacker(bio, use_list=True)
        for size in sizes:
            self.assertEqual(unpacker.unpack(), list(range(size)))

    def test_manual_reset(self):
        sizes = [0, 5, 50, 1000]
        packer = salt.utils.msgpack.Packer(autoreset=False)
        for size in sizes:
            packer.pack_array_header(size)
            for i in range(size):
                packer.pack(i)

        bio = BytesIO(packer.bytes())
        unpacker = salt.utils.msgpack.Unpacker(bio, use_list=True)
        for size in sizes:
            self.assertEqual(unpacker.unpack(), list(range(size)))

        packer.reset()
        self.assertEqual(packer.bytes(), b"")

    def test_map_size(self):
        sizes = [0, 5, 50, 1000]
        bio = BytesIO()
        packer = salt.utils.msgpack.Packer()
        for size in sizes:
            bio.write(packer.pack_map_header(size))
            for i in range(size):
                bio.write(packer.pack(i))  # key
                bio.write(packer.pack(i * 2))  # value

        bio.seek(0)
        if salt.utils.msgpack.version > (0, 6, 0):
            unpacker = salt.utils.msgpack.Unpacker(bio, strict_map_key=False)
        else:
            unpacker = salt.utils.msgpack.Unpacker(bio)
        for size in sizes:
            self.assertEqual(unpacker.unpack(), dict((i, i * 2) for i in range(size)))

    def test_exceptions(self):
        # Verify that this exception exists
        self.assertTrue(salt.utils.msgpack.exceptions.PackValueError)
        self.assertTrue(salt.utils.msgpack.exceptions.UnpackValueError)
        self.assertTrue(salt.utils.msgpack.exceptions.PackValueError)
        self.assertTrue(salt.utils.msgpack.exceptions.UnpackValueError)

    def test_function_aliases(self):
        """
        Fail if core functionality from msgpack is missing in the utility
        """

        def sanitized(item):
            if inspect.isfunction(getattr(msgpack, item)):
                # Only check objects that exist in the same file as msgpack
                return inspect.getfile(getattr(msgpack, item)) == inspect.getfile(
                    msgpack
                )

        msgpack_items = set(
            x for x in dir(msgpack) if not x.startswith("_") and sanitized(x)
        )
        msgpack_util_items = set(dir(salt.utils.msgpack))
        self.assertFalse(
            msgpack_items - msgpack_util_items,
            "msgpack functions with no alias in `salt.utils.msgpack`",
        )

    def test_sanitize_msgpack_kwargs(self):
        """
        Test helper function _sanitize_msgpack_kwargs
        """
        version = salt.utils.msgpack.version

        kwargs = {"strict_map_key": True, "raw": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 6, 0)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_kwargs(kwargs),
            {"raw": True, "strict_map_key": True, "use_bin_type": True},
        )

        kwargs = {"strict_map_key": True, "raw": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 5, 2)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_kwargs(kwargs),
            {"raw": True, "use_bin_type": True},
        )

        kwargs = {"strict_map_key": True, "raw": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 4, 0)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_kwargs(kwargs), {"use_bin_type": True}
        )

        kwargs = {"strict_map_key": True, "raw": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 3, 0)
        self.assertEqual(salt.utils.msgpack._sanitize_msgpack_kwargs(kwargs), {})
        salt.utils.msgpack.version = version

    def test_sanitize_msgpack_unpack_kwargs(self):
        """
        Test helper function _sanitize_msgpack_unpack_kwargs
        """
        version = salt.utils.msgpack.version

        kwargs = {"strict_map_key": True, "use_bin_type": True}
        salt.utils.msgpack.version = (1, 0, 0)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_unpack_kwargs(kwargs),
            {"raw": True, "strict_map_key": True, "use_bin_type": True},
        )

        kwargs = {"strict_map_key": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 6, 0)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_unpack_kwargs(kwargs),
            {"strict_map_key": True, "use_bin_type": True},
        )

        kwargs = {"strict_map_key": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 5, 2)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_unpack_kwargs(kwargs),
            {"use_bin_type": True},
        )

        kwargs = {"strict_map_key": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 4, 0)
        self.assertEqual(
            salt.utils.msgpack._sanitize_msgpack_unpack_kwargs(kwargs),
            {"use_bin_type": True},
        )
        kwargs = {"strict_map_key": True, "use_bin_type": True}
        salt.utils.msgpack.version = (0, 3, 0)
        self.assertEqual(salt.utils.msgpack._sanitize_msgpack_unpack_kwargs(kwargs), {})
        salt.utils.msgpack.version = version

    def _test_base(self, pack_func, unpack_func):
        """
        In msgpack, 'dumps' is an alias for 'packb' and 'loads' is an alias for 'unpackb'.
        Verify that both salt.utils.msgpack function variations pass the exact same test
        """
        data = os.urandom(1024)

        packed = pack_func(data)
        # Sanity Check
        self.assertTrue(packed)
        self.assertIsInstance(packed, bytes)
        self.assertNotEqual(data, packed)

        # Reverse the packing and the result should be equivalent to the original data
        unpacked = unpack_func(packed)
        self.assertEqual(data, unpacked)

    def _test_buffered_base(self, pack_func, unpack_func):
        data = os.urandom(1024).decode(errors="ignore")
        buffer = BytesIO()
        # Sanity check, we are not borking the BytesIO read function
        self.assertNotEqual(BytesIO.read, buffer.read)
        buffer.read = buffer.getvalue

        pack_func(data, buffer)
        # Sanity Check
        self.assertTrue(buffer.getvalue())
        self.assertIsInstance(buffer.getvalue(), bytes)
        self.assertNotEqual(data, buffer.getvalue())

        # Reverse the packing and the result should be equivalent to the original data
        unpacked = unpack_func(buffer)
        self.assertEqual(data, unpacked.decode())

    def test_buffered_base_pack(self):
        self._test_buffered_base(
            pack_func=salt.utils.msgpack.pack, unpack_func=msgpack.unpack
        )

    def test_buffered_base_unpack(self):
        self._test_buffered_base(
            pack_func=msgpack.pack, unpack_func=salt.utils.msgpack.unpack
        )

    def _test_unpack_array_header_from_file(self, pack_func, **kwargs):
        f = BytesIO(pack_func([1, 2, 3, 4]))
        unpacker = salt.utils.msgpack.Unpacker(f)
        self.assertEqual(unpacker.read_array_header(), 4)
        self.assertEqual(unpacker.unpack(), 1)
        self.assertEqual(unpacker.unpack(), 2)
        self.assertEqual(unpacker.unpack(), 3)
        self.assertEqual(unpacker.unpack(), 4)
        self.assertRaises(salt.utils.msgpack.exceptions.OutOfData, unpacker.unpack)

    @skipIf(
        not hasattr(sys, "getrefcount"), "sys.getrefcount() is needed to pass this test"
    )
    def _test_unpacker_hook_refcnt(self, pack_func, **kwargs):
        result = []

        def hook(x):
            result.append(x)
            return x

        basecnt = sys.getrefcount(hook)

        up = salt.utils.msgpack.Unpacker(object_hook=hook, list_hook=hook)

        self.assertGreaterEqual(sys.getrefcount(hook), basecnt + 2)

        up.feed(pack_func([{}]))
        up.feed(pack_func([{}]))
        self.assertEqual(up.unpack(), [{}])
        self.assertEqual(up.unpack(), [{}])
        self.assertEqual(result, [{}, [{}], {}, [{}]])

        del up

        self.assertEqual(sys.getrefcount(hook), basecnt)

    def _test_unpacker_ext_hook(self, pack_func, **kwargs):
        class MyUnpacker(salt.utils.msgpack.Unpacker):
            def __init__(self):
                my_kwargs = {}
                super(MyUnpacker, self).__init__(ext_hook=self._hook, **raw)

            def _hook(self, code, data):
                if code == 1:
                    return int(data)
                else:
                    return salt.utils.msgpack.ExtType(code, data)

        unpacker = MyUnpacker()
        unpacker.feed(pack_func({"a": 1}))
        self.assertEqual(unpacker.unpack(), {"a": 1})
        unpacker.feed(pack_func({"a": salt.utils.msgpack.ExtType(1, b"123")}))
        self.assertEqual(unpacker.unpack(), {"a": 123})
        unpacker.feed(pack_func({"a": salt.utils.msgpack.ExtType(2, b"321")}))
        self.assertEqual(
            unpacker.unpack(), {"a": salt.utils.msgpack.ExtType(2, b"321")}
        )

    def _check(
        self, data, pack_func, unpack_func, use_list=False, strict_map_key=False
    ):
        my_kwargs = {}
        if salt.utils.msgpack.version >= (0, 6, 0):
            my_kwargs["strict_map_key"] = strict_map_key
        ret = unpack_func(pack_func(data), use_list=use_list, **my_kwargs)
        self.assertEqual(ret, data)

    def _test_pack_unicode(self, pack_func, unpack_func):
        test_data = [u"", u"abcd", [u"defgh"], u"Русский текст"]
        for td in test_data:
            ret = unpack_func(pack_func(td), use_list=True, **raw)
            self.assertEqual(ret, td)
            packer = salt.utils.msgpack.Packer()
            data = packer.pack(td)
            ret = salt.utils.msgpack.Unpacker(
                BytesIO(data), use_list=True, **raw
            ).unpack()
            self.assertEqual(ret, td)

    def _test_pack_bytes(self, pack_func, unpack_func):
        test_data = [
            b"",
            b"abcd",
            (b"defgh",),
        ]
        for td in test_data:
            self._check(td, pack_func, unpack_func)

    def _test_pack_byte_arrays(self, pack_func, unpack_func):
        test_data = [
            bytearray(b""),
            bytearray(b"abcd"),
            (bytearray(b"defgh"),),
        ]
        for td in test_data:
            self._check(td, pack_func, unpack_func)

    @skipIf(sys.version_info < (3, 0), "Python 2 passes invalid surrogates")
    def _test_ignore_unicode_errors(self, pack_func, unpack_func):
        ret = unpack_func(
            pack_func(b"abc\xeddef", use_bin_type=False), unicode_errors="ignore", **raw
        )
        self.assertEqual(u"abcdef", ret)

    def _test_strict_unicode_unpack(self, pack_func, unpack_func):
        packed = pack_func(b"abc\xeddef", use_bin_type=False)
        self.assertRaises(UnicodeDecodeError, unpack_func, packed, use_list=True, **raw)

    @skipIf(sys.version_info < (3, 0), "Python 2 passes invalid surrogates")
    def _test_ignore_errors_pack(self, pack_func, unpack_func):
        ret = unpack_func(
            pack_func(
                u"abc\uDC80\uDCFFdef", use_bin_type=True, unicode_errors="ignore"
            ),
            use_list=True,
            **raw
        )
        self.assertEqual(u"abcdef", ret)

    def _test_decode_binary(self, pack_func, unpack_func):
        ret = unpack_func(pack_func(b"abc"), use_list=True)
        self.assertEqual(b"abc", ret)

    @skipIf(
        salt.utils.msgpack.version < (0, 2, 2),
        "use_single_float was added in msgpack==0.2.2",
    )
    def _test_pack_float(self, pack_func, **kwargs):
        self.assertEqual(
            b"\xca" + struct.pack(str(">f"), 1.0), pack_func(1.0, use_single_float=True)
        )
        self.assertEqual(
            b"\xcb" + struct.pack(str(">d"), 1.0),
            pack_func(1.0, use_single_float=False),
        )

    def _test_odict(self, pack_func, unpack_func):
        seq = [(b"one", 1), (b"two", 2), (b"three", 3), (b"four", 4)]

        od = OrderedDict(seq)
        self.assertEqual(dict(seq), unpack_func(pack_func(od), use_list=True))

        def pair_hook(seq):
            return list(seq)

        self.assertEqual(
            seq, unpack_func(pack_func(od), object_pairs_hook=pair_hook, use_list=True)
        )

    def _test_pair_list(self, unpack_func, **kwargs):
        pairlist = [(b"a", 1), (2, b"b"), (b"foo", b"bar")]
        packer = salt.utils.msgpack.Packer()
        packed = packer.pack_map_pairs(pairlist)
        if salt.utils.msgpack.version > (0, 6, 0):
            unpacked = unpack_func(packed, object_pairs_hook=list, strict_map_key=False)
        else:
            unpacked = unpack_func(packed, object_pairs_hook=list)
        self.assertEqual(pairlist, unpacked)

    @skipIf(
        salt.utils.msgpack.version < (0, 6, 0),
        "getbuffer() was added to Packer in msgpack 0.6.0",
    )
    def _test_get_buffer(self, pack_func, **kwargs):
        packer = msgpack.Packer(autoreset=False, use_bin_type=True)
        packer.pack([1, 2])
        strm = BytesIO()
        strm.write(packer.getbuffer())
        written = strm.getvalue()

        expected = pack_func([1, 2], use_bin_type=True)
        self.assertEqual(expected, written)

    @staticmethod
    def no_fail_run(test, *args, **kwargs):
        """
        Run a test without failure and return any exception it raises
        """
        try:
            test(*args, **kwargs)
        except Exception as e:  # pylint: disable=broad-except
            return e

    def test_binary_function_compatibility(self):
        functions = [
            {"pack_func": salt.utils.msgpack.packb, "unpack_func": msgpack.unpackb},
            {"pack_func": msgpack.packb, "unpack_func": salt.utils.msgpack.unpackb},
        ]
        # These functions are equivalent but could potentially be overwritten
        if salt.utils.msgpack.dumps is not salt.utils.msgpack.packb:
            functions.append(
                {"pack_func": salt.utils.msgpack.dumps, "unpack_func": msgpack.unpackb}
            )
        if salt.utils.msgpack.loads is not salt.utils.msgpack.unpackb:
            functions.append(
                {"pack_func": msgpack.packb, "unpack_func": salt.utils.msgpack.loads}
            )

        test_funcs = (
            self._test_base,
            self._test_unpack_array_header_from_file,
            self._test_unpacker_hook_refcnt,
            self._test_unpacker_ext_hook,
            self._test_pack_unicode,
            self._test_pack_bytes,
            self._test_pack_byte_arrays,
            self._test_ignore_unicode_errors,
            self._test_strict_unicode_unpack,
            self._test_ignore_errors_pack,
            self._test_decode_binary,
            self._test_pack_float,
            self._test_odict,
            self._test_pair_list,
            self._test_get_buffer,
        )
        errors = {}
        for test_func in test_funcs:
            # Run the test without the salt.utils.msgpack module for comparison
            vanilla_run = self.no_fail_run(
                test_func,
                **{"pack_func": msgpack.packb, "unpack_func": msgpack.unpackb}
            )

            for func_args in functions:
                func_name = (
                    func_args["pack_func"]
                    if func_args["pack_func"].__module__.startswith("salt.utils")
                    else func_args["unpack_func"]
                )
                if hasattr(TestCase, "subTest"):
                    with self.subTest(test=test_func.__name__, func=func_name.__name__):
                        # Run the test with the salt.utils.msgpack module
                        run = self.no_fail_run(test_func, **func_args)
                        # If the vanilla msgpack module errored, then skip if we got the same error
                        if run:
                            if str(vanilla_run) == str(run):
                                self.skipTest(
                                    "Failed the same way as the vanilla msgpack module:\n{}".format(
                                        run
                                    )
                                )
                else:
                    # If subTest isn't available then run the tests collect the errors of all the tests before failing
                    run = self.no_fail_run(test_func, **func_args)
                    if run:
                        # If the vanilla msgpack module errored, then skip if we got the same error
                        if str(vanilla_run) == str(run):
                            self.skipTest(
                                "Test failed the same way the vanilla msgpack module fails:\n{}".format(
                                    run
                                )
                            )
                        else:
                            errors[(test_func.__name__, func_name.__name__)] = run

        if errors:
            self.fail(pprint.pformat(errors))



# -*- coding: utf-8 -*-
"""
unit tests for the grains state
"""

from __future__ import absolute_import, print_function, unicode_literals

import contextlib

# Import Python libs
import os

import salt.modules.grains as grainsmod
import salt.states.grains as grains

# Import salt libs
import salt.utils.files
import salt.utils.stringutils
import salt.utils.yaml
from salt.ext import six
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase


class GrainsTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        grains_test_dir = "__salt_test_state_grains"
        if not os.path.exists(os.path.join(RUNTIME_VARS.TMP, grains_test_dir)):
            os.makedirs(os.path.join(RUNTIME_VARS.TMP, grains_test_dir))
        loader_globals = {
            "__opts__": {
                "test": False,
                "conf_file": os.path.join(RUNTIME_VARS.TMP, grains_test_dir, "minion"),
                "cachedir": os.path.join(RUNTIME_VARS.TMP, grains_test_dir),
                "local": True,
            },
            "__salt__": {
                "cmd.run_all": MagicMock(
                    return_value={"pid": 5, "retcode": 0, "stderr": "", "stdout": ""}
                ),
                "grains.get": grainsmod.get,
                "grains.set": grainsmod.set,
                "grains.setval": grainsmod.setval,
                "grains.delval": grainsmod.delval,
                "grains.append": grainsmod.append,
                "grains.remove": grainsmod.remove,
                "saltutil.sync_grains": MagicMock(),
            },
        }
        return {grains: loader_globals, grainsmod: loader_globals}

    def assertGrainFileContent(self, grains_string):
        if os.path.isdir(grains.__opts__["conf_file"]):
            grains_file = os.path.join(grains.__opts__["conf_file"], "grains")
        else:
            grains_file = os.path.join(
                os.path.dirname(grains.__opts__["conf_file"]), "grains"
            )
        with salt.utils.files.fopen(grains_file, "r") as grf:
            grains_data = salt.utils.stringutils.to_unicode(grf.read())
        self.assertMultiLineEqual(grains_string, grains_data)

    @contextlib.contextmanager
    def setGrains(self, grains_data):
        with patch.dict(grains.__grains__, grains_data):
            with patch.dict(grainsmod.__grains__, grains_data):
                if os.path.isdir(grains.__opts__["conf_file"]):
                    grains_file = os.path.join(grains.__opts__["conf_file"], "grains")
                else:
                    grains_file = os.path.join(
                        os.path.dirname(grains.__opts__["conf_file"]), "grains"
                    )
                with salt.utils.files.fopen(grains_file, "w+") as grf:
                    salt.utils.yaml.safe_dump(
                        grains_data, grf, default_flow_style=False
                    )
                yield

    # 'exists' function tests: 2

    def test_exists_missing(self):
        with self.setGrains({"a": "aval"}):
            ret = grains.exists(name="foo")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "Grain does not exist")
            self.assertEqual(ret["changes"], {})

    def test_exists_found(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Grain already set
            ret = grains.exists(name="foo")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain exists")
            self.assertEqual(ret["changes"], {})

    # 'make_hashable' function tests: 1

    def test_make_hashable(self):
        with self.setGrains({"cmplx_lst_grain": [{"a": "aval"}, {"foo": "bar"}]}):
            hashable_list = {"cmplx_lst_grain": [{"a": "aval"}, {"foo": "bar"}]}
            self.assertEqual(
                grains.make_hashable(grains.__grains__).issubset(
                    grains.make_hashable(hashable_list)
                ),
                True,
            )

    # 'present' function tests: 12

    def test_present_add(self):
        # Set a non existing grain
        with self.setGrains({"a": "aval"}):
            ret = grains.present(name="foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": "bar"})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
            self.assertGrainFileContent("a: aval\nfoo: bar\n")

        # Set a non existing nested grain
        with self.setGrains({"a": "aval"}):
            ret = grains.present(name="foo:is:nested", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": {"is": {"nested": "bar"}}})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "bar"}}}
            )
            self.assertGrainFileContent(
                "a: aval\n" "foo:\n" "  is:\n" "    nested: bar\n"
            )

        # Set a non existing nested dict grain
        with self.setGrains({"a": "aval"}):
            ret = grains.present(name="foo:is:nested", value={"bar": "is a dict"})
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["changes"], {"foo": {"is": {"nested": {"bar": "is a dict"}}}}
            )
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": {"is": {"nested": {"bar": "is a dict"}}}},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "  is:\n"
                + "    nested:\n"
                + "      bar: is a dict\n"
            )

    def test_present_add_key_to_existing(self):
        with self.setGrains({"a": "aval", "foo": {"k1": "v1"}}):
            # Fails setting a grain to a dict
            ret = grains.present(name="foo:k2", value="v2")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Set grain foo:k2 to v2")
            self.assertEqual(ret["changes"], {"foo": {"k2": "v2", "k1": "v1"}})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"k1": "v1", "k2": "v2"}}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  k1: v1\n" + "  k2: v2\n"
            )

    def test_present_already_set(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Grain already set
            ret = grains.present(name="foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain is already set")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Nested grain already set
            ret = grains.present(name="foo:is:nested", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain is already set")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "bar"}}}
            )

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Nested dict grain already set
            ret = grains.present(name="foo:is", value={"nested": "bar"})
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain is already set")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "bar"}}}
            )

    def test_present_overwrite(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Overwrite an existing grain
            ret = grains.present(name="foo", value="newbar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": "newbar"})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "newbar"})
            self.assertGrainFileContent("a: aval\n" + "foo: newbar\n")

        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Clear a grain (set to None)
            ret = grains.present(name="foo", value=None)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": None})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": None})
            self.assertGrainFileContent("a: aval\n" + "foo: null\n")

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Overwrite an existing nested grain
            ret = grains.present(name="foo:is:nested", value="newbar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": {"is": {"nested": "newbar"}}})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "newbar"}}}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  is:\n" + "    nested: newbar\n"
            )

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Clear a nested grain (set to None)
            ret = grains.present(name="foo:is:nested", value=None)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": {"is": {"nested": None}}})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": None}}}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  is:\n" + "    nested: null\n"
            )

    def test_present_fail_overwrite(self):
        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "val"}}}):
            # Overwrite an existing grain
            ret = grains.present(name="foo:is", value="newbar")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                ret["comment"],
                "The key 'foo:is' exists but is a dict or a list. Use 'force=True' to overwrite.",
            )
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "val"}}}
            )

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "val"}}}):
            # Clear a grain (set to None)
            ret = grains.present(name="foo:is", value=None)
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                ret["comment"],
                "The key 'foo:is' exists but is a dict or a list. Use 'force=True' to overwrite.",
            )
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "val"}}}
            )

    def test_present_fails_to_set_dict_or_list(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Fails to overwrite a grain to a list
            ret = grains.present(name="foo", value=["l1", "l2"])
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo' exists and the "
                + "given value is a dict or a list. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})

        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Fails setting a grain to a dict
            ret = grains.present(name="foo", value={"k1": "v1"})
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo' exists and the given "
                + "value is a dict or a list. Use "
                + "'force=True' to overwrite.",
            )
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Fails to overwrite a nested grain to a list
            ret = grains.present(
                name="foo,is,nested", value=["l1", "l2"], delimiter=","
            )
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                ret["comment"],
                "The key 'foo:is:nested' exists and the "
                + "given value is a dict or a list. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "bar"}}}
            )

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Fails setting a nested grain to a dict
            ret = grains.present(name="foo:is:nested", value={"k1": "v1"})
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo:is:nested' exists and the "
                + "given value is a dict or a list. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": "bar"}}}
            )

    def test_present_fail_merge_dict(self):
        with self.setGrains({"a": "aval", "foo": {"k1": "v1"}}):
            # Fails setting a grain to a dict
            ret = grains.present(name="foo", value={"k2": "v2"})
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo' exists but "
                + "is a dict or a list. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": {"k1": "v1"}})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "  k1: v1\n")

    def test_present_force_to_set_dict_or_list(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Force to overwrite a grain to a list
            ret = grains.present(name="foo", value=["l1", "l2"], force=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"],
                "Set grain foo to ['l1', 'l2']"
                if six.PY3
                else "Set grain foo to [u'l1', u'l2']",
            )
            self.assertEqual(ret["changes"], {"foo": ["l1", "l2"]})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["l1", "l2"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- l1\n" + "- l2\n")

        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Force setting a grain to a dict
            ret = grains.present(name="foo", value={"k1": "v1"}, force=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"],
                "Set grain foo to {'k1': 'v1'}"
                if six.PY3
                else "Set grain foo to {u'k1': u'v1'}",
            )
            self.assertEqual(ret["changes"], {"foo": {"k1": "v1"}})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": {"k1": "v1"}})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "  k1: v1\n")

        with self.setGrains({"a": "aval", "foo": {"is": {"nested": "bar"}}}):
            # Force to overwrite a nested grain to a list
            ret = grains.present(
                name="foo,is,nested", value=["l1", "l2"], delimiter=",", force=True
            )
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["changes"], {"foo": {"is": {"nested": ["l1", "l2"]}}})
            self.assertEqual(
                ret["comment"],
                "Set grain foo:is:nested to ['l1', 'l2']"
                if six.PY3
                else "Set grain foo:is:nested to [u'l1', u'l2']",
            )
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": {"is": {"nested": ["l1", "l2"]}}},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "  is:\n"
                + "    nested:\n"
                + "    - l1\n"
                + "    - l2\n"
            )

        with self.setGrains(
            {"a": "aval", "foo": {"is": {"nested": "bar"}, "and": "other"}}
        ):
            # Force setting a nested grain to a dict
            ret = grains.present(name="foo:is:nested", value={"k1": "v1"}, force=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"],
                "Set grain foo:is:nested to {'k1': 'v1'}"
                if six.PY3
                else "Set grain foo:is:nested to {u'k1': u'v1'}",
            )
            self.assertEqual(
                ret["changes"],
                {"foo": {"is": {"nested": {"k1": "v1"}}, "and": "other"}},
            )
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": {"is": {"nested": {"k1": "v1"}}, "and": "other"}},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "  and: other\n"
                + "  is:\n"
                + "    nested:\n"
                + "      k1: v1\n"
            )

    def test_present_fails_to_convert_value_to_key(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            # Fails converting a value to a nested grain key
            ret = grains.present(name="foo:is:nested", value={"k1": "v1"})
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo' value is 'bar', "
                + "which is different from the provided "
                + "key 'is'. Use 'force=True' to overwrite.",
            )
            self.assertEqual(ret["changes"], {})

    def test_present_overwrite_test(self):
        with patch.dict(grains.__opts__, {"test": True}):
            with self.setGrains({"a": "aval", "foo": "bar"}):
                # Overwrite an existing grain
                ret = grains.present(name="foo", value="newbar")
                self.assertEqual(ret["result"], None)
                self.assertEqual(ret["changes"], {"changed": {"foo": "newbar"}})
                self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
                self.assertGrainFileContent("a: aval\n" + "foo: bar\n")

    def test_present_convert_value_to_key(self):
        with self.setGrains({"a": "aval", "foo": "is"}):
            # Converts a value to a nested grain key
            ret = grains.present(name="foo:is:nested", value={"k1": "v1"})
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"],
                "Set grain foo:is:nested to {'k1': 'v1'}"
                if six.PY3
                else "Set grain foo:is:nested to {u'k1': u'v1'}",
            )
            self.assertEqual(ret["changes"], {"foo": {"is": {"nested": {"k1": "v1"}}}})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": {"is": {"nested": {"k1": "v1"}}}},
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  is:\n" + "    nested:\n" + "      k1: v1\n"
            )

        with self.setGrains({"a": "aval", "foo": ["one", "is", "correct"]}):
            # Converts a list element to a nested grain key
            ret = grains.present(name="foo:is:nested", value={"k1": "v1"})
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"],
                "Set grain foo:is:nested to {'k1': 'v1'}"
                if six.PY3
                else "Set grain foo:is:nested to {u'k1': u'v1'}",
            )
            self.assertEqual(
                ret["changes"],
                {"foo": ["one", {"is": {"nested": {"k1": "v1"}}}, "correct"]},
            )
            self.assertEqual(
                grains.__grains__,
                {
                    "a": "aval",
                    "foo": ["one", {"is": {"nested": {"k1": "v1"}}}, "correct"],
                },
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "- one\n"
                + "- is:\n"
                + "    nested:\n"
                + "      k1: v1\n"
                + "- correct\n"
            )

    def test_present_unknown_failure(self):
        with patch("salt.modules.grains.setval") as mocked_setval:
            mocked_setval.return_value = "Failed to set grain foo"
            with self.setGrains({"a": "aval", "foo": "bar"}):
                # Unknown reason failure
                ret = grains.present(name="foo", value="baz")
                self.assertEqual(ret["result"], False)
                self.assertEqual(ret["comment"], "Failed to set grain foo")
                self.assertEqual(ret["changes"], {})
                self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
                self.assertGrainFileContent("a: aval\n" + "foo: bar\n")

    # 'absent' function tests: 6

    def test_absent_already(self):
        # Unset a non existent grain
        with self.setGrains({"a": "aval"}):
            ret = grains.absent(name="foo")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

        # Unset a non existent nested grain
        with self.setGrains({"a": "aval"}):
            ret = grains.absent(name="foo:is:nested")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo:is:nested does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

    def test_absent_unset(self):
        # Unset a grain
        with self.setGrains({"a": "aval", "foo": "bar"}):
            ret = grains.absent(name="foo")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value for grain foo was set to None")
            self.assertEqual(ret["changes"], {"grain": "foo", "value": None})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": None})
            self.assertGrainFileContent("a: aval\n" + "foo: null\n")

        # Unset grain when its value is False
        with self.setGrains({"a": "aval", "foo": False}):
            ret = grains.absent(name="foo")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value for grain foo was set to None")
            self.assertEqual(ret["changes"], {"grain": "foo", "value": None})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": None})
            self.assertGrainFileContent("a: aval\n" + "foo: null\n")

        # Unset a nested grain
        with self.setGrains(
            {"a": "aval", "foo": ["order", {"is": {"nested": "bar"}}, "correct"]}
        ):
            ret = grains.absent(name="foo,is,nested", delimiter=",")
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"], "Value for grain foo:is:nested was set to None"
            )
            self.assertEqual(ret["changes"], {"grain": "foo:is:nested", "value": None})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": ["order", {"is": {"nested": None}}, "correct"]},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "- order\n"
                + "- is:\n"
                + "    nested: null\n"
                + "- correct\n"
            )

        # Unset a nested value don't change anything
        with self.setGrains(
            {"a": "aval", "foo": ["order", {"is": "nested"}, "correct"]}
        ):
            ret = grains.absent(name="foo:is:nested")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo:is:nested does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": ["order", {"is": "nested"}, "correct"]},
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "- order\n" + "- is: nested\n" + "- correct\n"
            )

    def test_absent_unset_test(self):
        with patch.dict(grains.__opts__, {"test": True}):
            with self.setGrains({"a": "aval", "foo": "bar"}):
                # Overwrite an existing grain
                ret = grains.absent(name="foo")
                self.assertEqual(ret["result"], None)
                self.assertEqual(ret["changes"], {"grain": "foo", "value": None})
                self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
                self.assertGrainFileContent("a: aval\n" + "foo: bar\n")

    def test_absent_fails_nested_complex_grain(self):
        # Unset a nested complex grain
        with self.setGrains(
            {"a": "aval", "foo": ["order", {"is": {"nested": "bar"}}, "correct"]}
        ):
            ret = grains.absent(name="foo:is")
            self.assertEqual(ret["result"], False)
            self.assertEqual(
                ret["comment"],
                "The key 'foo:is' exists but is a dict or a list. Use 'force=True' to overwrite.",
            )
            self.assertEqual(ret["changes"], {})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": ["order", {"is": {"nested": "bar"}}, "correct"]},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "- order\n"
                + "- is:\n"
                + "    nested: bar\n"
                + "- correct\n"
            )

    def test_absent_force_nested_complex_grain(self):
        # Unset a nested complex grain
        with self.setGrains(
            {"a": "aval", "foo": ["order", {"is": {"nested": "bar"}}, "correct"]}
        ):
            ret = grains.absent(name="foo:is", force=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value for grain foo:is was set to None")
            self.assertEqual(ret["changes"], {"grain": "foo:is", "value": None})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": ["order", {"is": None}, "correct"]},
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "- order\n" + "- is: null\n" + "- correct\n"
            )

    def test_absent_delete(self):
        # Delete a grain
        with self.setGrains({"a": "aval", "foo": "bar"}):
            ret = grains.absent(name="foo", destructive=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo was deleted")
            self.assertEqual(ret["changes"], {"deleted": "foo"})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

        # Delete a previously unset grain
        with self.setGrains({"a": "aval", "foo": None}):
            ret = grains.absent(name="foo", destructive=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo was deleted")
            self.assertEqual(ret["changes"], {"deleted": "foo"})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

        # Delete a nested grain
        with self.setGrains(
            {
                "a": "aval",
                "foo": [
                    "order",
                    {"is": {"nested": "bar", "other": "value"}},
                    "correct",
                ],
            }
        ):
            ret = grains.absent(name="foo:is:nested", destructive=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo:is:nested was deleted")
            self.assertEqual(ret["changes"], {"deleted": "foo:is:nested"})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": ["order", {"is": {"other": "value"}}, "correct"]},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "- order\n"
                + "- is:\n"
                + "    other: value\n"
                + "- correct\n"
            )

    # 'append' function tests: 6

    def test_append(self):
        # Append to an existing list
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.append(name="foo", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz was added to grain foo")
            self.assertEqual(ret["changes"], {"added": "baz"})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar", "baz"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n" + "- baz\n")

    def test_append_nested(self):
        # Append to an existing nested list
        with self.setGrains({"a": "aval", "foo": {"list": ["bar"]}}):
            ret = grains.append(name="foo,list", value="baz", delimiter=",")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz was added to grain foo:list")
            self.assertEqual(ret["changes"], {"added": "baz"})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"list": ["bar", "baz"]}}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  list:\n" + "  - bar\n" + "  - baz\n"
            )

    def test_append_already(self):
        # Append to an existing list
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.append(name="foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"], "Value bar is already in the list " + "for grain foo"
            )
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n")

    def test_append_fails_not_a_list(self):
        # Fail to append to an existing grain, not a list
        with self.setGrains({"a": "aval", "foo": {"bar": "val"}}):
            ret = grains.append(name="foo", value="baz")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "Grain foo is not a valid list")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": {"bar": "val"}})

    def test_append_convert_to_list(self):
        # Append to an existing grain, converting to a list
        with self.setGrains({"a": "aval", "foo": {"bar": "val"}}):
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "  bar: val\n")
            ret = grains.append(name="foo", value="baz", convert=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz was added to grain foo")
            self.assertEqual(ret["changes"], {"added": "baz"})
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": [{"bar": "val"}, "baz"]}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "- bar: val\n" + "- baz\n"
            )

        # Append to an existing grain, converting to a list a multi-value dict
        with self.setGrains({"a": "aval", "foo": {"bar": "val", "other": "value"}}):
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  bar: val\n" + "  other: value\n"
            )
            ret = grains.append(name="foo", value="baz", convert=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz was added to grain foo")
            self.assertEqual(ret["changes"], {"added": "baz"})
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": [{"bar": "val", "other": "value"}, "baz"]},
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "- bar: val\n" + "  other: value\n" + "- baz\n"
            )

    def test_append_fails_inexistent(self):
        # Append to a non existing grain
        with self.setGrains({"a": "aval"}):
            ret = grains.append(name="foo", value="bar")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "Grain foo does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval"})

    def test_append_convert_to_list_empty(self):
        # Append to an existing list
        with self.setGrains({"foo": None}):
            ret = grains.append(name="foo", value="baz", convert=True)
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz was added to grain foo")
            self.assertEqual(ret["changes"], {"added": "baz"})
            self.assertEqual(grains.__grains__, {"foo": ["baz"]})
            self.assertGrainFileContent("foo:\n" + "- baz\n")

    # 'list_present' function tests: 7

    def test_list_present(self):
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.list_present(name="foo", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Append value baz to grain foo")
            self.assertEqual(ret["changes"], {"new": {"foo": ["bar", "baz"]}})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar", "baz"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n" + "- baz\n")

    def test_list_present_nested(self):
        with self.setGrains({"a": "aval", "foo": {"is": {"nested": ["bar"]}}}):
            ret = grains.list_present(name="foo,is,nested", value="baz", delimiter=",")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Append value baz to grain foo:is:nested")
            self.assertEqual(
                ret["changes"], {"new": {"foo": {"is": {"nested": ["bar", "baz"]}}}}
            )
            self.assertEqual(
                grains.__grains__,
                {"a": "aval", "foo": {"is": {"nested": ["bar", "baz"]}}},
            )
            self.assertGrainFileContent(
                "a: aval\n"
                + "foo:\n"
                + "  is:\n"
                + "    nested:\n"
                + "    - bar\n"
                + "    - baz\n"
            )

    def test_list_present_inexistent(self):
        with self.setGrains({"a": "aval"}):
            ret = grains.list_present(name="foo", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Append value baz to grain foo")
            self.assertEqual(ret["changes"], {"new": {"foo": ["baz"]}})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["baz"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- baz\n")

    def test_list_present_inexistent_nested(self):
        with self.setGrains({"a": "aval"}):
            ret = grains.list_present(name="foo:is:nested", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Append value baz to grain foo:is:nested")
            self.assertEqual(
                ret["changes"], {"new": {"foo": {"is": {"nested": ["baz"]}}}}
            )
            self.assertEqual(
                grains.__grains__, {"a": "aval", "foo": {"is": {"nested": ["baz"]}}}
            )
            self.assertGrainFileContent(
                "a: aval\n" + "foo:\n" + "  is:\n" + "    nested:\n" + "    - baz\n"
            )

    def test_list_present_not_a_list(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            ret = grains.list_present(name="foo", value="baz")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "Grain foo is not a valid list")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
            self.assertGrainFileContent("a: aval\n" + "foo: bar\n")

    def test_list_present_nested_already(self):
        with self.setGrains({"a": "aval", "b": {"foo": ["bar"]}}):
            ret = grains.list_present(name="b:foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value bar is already in grain b:foo")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "b": {"foo": ["bar"]}})
            self.assertGrainFileContent("a: aval\n" + "b:\n" + "  foo:\n" + "  - bar\n")

    def test_list_present_already(self):
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.list_present(name="foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value bar is already in grain foo")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n")

    def test_list_present_unknown_failure(self):
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            # Unknown reason failure

            with patch.dict(grainsmod.__salt__, {"grains.append": MagicMock()}):
                ret = grains.list_present(name="foo", value="baz")
                self.assertEqual(ret["result"], False)
                self.assertEqual(ret["comment"], "Failed append value baz to grain foo")
                self.assertEqual(ret["changes"], {})
                self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar"]})
                self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n")

    # 'list_absent' function tests: 6

    def test_list_absent(self):
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.list_absent(name="foo", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value bar was deleted from grain foo")
            self.assertEqual(ret["changes"], {"deleted": ["bar"]})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": []})
            self.assertGrainFileContent("a: aval\n" + "foo: []\n")

    def test_list_absent_nested(self):
        with self.setGrains({"a": "aval", "foo": {"list": ["bar"]}}):
            ret = grains.list_absent(name="foo:list", value="bar")
            self.assertEqual(ret["result"], True)
            self.assertEqual(
                ret["comment"], "Value bar was deleted from grain foo:list"
            )
            self.assertEqual(ret["changes"], {"deleted": ["bar"]})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": {"list": []}})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "  list: []\n")

    def test_list_absent_inexistent(self):
        with self.setGrains({"a": "aval"}):
            ret = grains.list_absent(name="foo", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

    def test_list_absent_inexistent_nested(self):
        with self.setGrains({"a": "aval"}):
            ret = grains.list_absent(name="foo:list", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Grain foo:list does not exist")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval"})
            self.assertGrainFileContent("a: aval\n")

    def test_list_absent_not_a_list(self):
        with self.setGrains({"a": "aval", "foo": "bar"}):
            ret = grains.list_absent(name="foo", value="bar")
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "Grain foo is not a valid list")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": "bar"})
            self.assertGrainFileContent("a: aval\n" + "foo: bar\n")

    def test_list_absent_already(self):
        with self.setGrains({"a": "aval", "foo": ["bar"]}):
            ret = grains.list_absent(name="foo", value="baz")
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], "Value baz is absent from grain foo")
            self.assertEqual(ret["changes"], {})
            self.assertEqual(grains.__grains__, {"a": "aval", "foo": ["bar"]})
            self.assertGrainFileContent("a: aval\n" + "foo:\n" + "- bar\n")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.modjk_worker as modjk_worker

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ModjkWorkerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.modjk_worker
    """

    def setup_loader_modules(self):
        return {modjk_worker: {}}

    # 'stop' function tests: 1

    def test_stop(self):
        """
        Test to stop the named worker from the lbn load balancers
         at the targeted minions.
        """
        name = "{{ grains['id'] }}"
        lbn = "application"
        target = "roles:balancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        comt = "no servers answered the published command modjk.worker_status"
        mock = MagicMock(return_value=False)
        with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
            ret.update({"comment": comt})
            self.assertDictEqual(modjk_worker.stop(name, lbn, target), ret)

    # 'activate' function tests: 1

    def test_activate(self):
        """
        Test to activate the named worker from the lbn load balancers
         at the targeted minions.
        """
        name = "{{ grains['id'] }}"
        lbn = "application"
        target = "roles:balancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        comt = "no servers answered the published command modjk.worker_status"
        mock = MagicMock(return_value=False)
        with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
            ret.update({"comment": comt})
            self.assertDictEqual(modjk_worker.activate(name, lbn, target), ret)

    # 'disable' function tests: 1

    def test_disable(self):
        """
        Test to disable the named worker from the lbn load balancers
         at the targeted minions.
        """
        name = "{{ grains['id'] }}"
        lbn = "application"
        target = "roles:balancer"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        comt = "no servers answered the published command modjk.worker_status"
        mock = MagicMock(return_value=False)
        with patch.dict(modjk_worker.__salt__, {"publish.publish": mock}):
            ret.update({"comment": comt})
            self.assertDictEqual(modjk_worker.disable(name, lbn, target), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Jeff Schroeder <jeffschroeder@computer.org>`
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import base64
from contextlib import contextmanager

# Import Salt Libs
import salt.utils.stringutils
from salt.ext import six
from salt.states import kubernetes

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


@skipIf(
    kubernetes is False,
    "Probably Kubernetes client lib is not installed. \
                              Skipping test_kubernetes.py",
)
class KubernetesTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.kubernetes
    """

    def setup_loader_modules(self):
        return {kubernetes: {"__env__": "base"}}

    @contextmanager
    def mock_func(self, func_name, return_value, test=False):
        """
        Mock any of the kubernetes state function return values and set
        the test options.
        """
        name = "kubernetes.{0}".format(func_name)
        mocked = {name: MagicMock(return_value=return_value)}
        with patch.dict(kubernetes.__salt__, mocked) as patched:
            with patch.dict(kubernetes.__opts__, {"test": test}):
                yield patched

    def make_configmap(self, name, namespace="default", data=None):
        return self.make_ret_dict(
            kind="ConfigMap", name=name, namespace=namespace, data=data,
        )

    def make_secret(self, name, namespace="default", data=None):
        secret_data = self.make_ret_dict(
            kind="Secret", name=name, namespace=namespace, data=data,
        )
        # Base64 all of the values just like kubectl does
        for key, value in six.iteritems(secret_data["data"]):
            secret_data["data"][key] = base64.b64encode(
                salt.utils.stringutils.to_bytes(value)
            )

        return secret_data

    def make_node_labels(self, name="minikube"):
        return {
            "kubernetes.io/hostname": name,
            "beta.kubernetes.io/os": "linux",
            "beta.kubernetes.io/arch": "amd64",
            "failure-domain.beta.kubernetes.io/region": "us-west-1",
        }

    def make_node(self, name="minikube"):
        node_data = self.make_ret_dict(kind="Node", name="minikube")
        node_data.update(
            {
                "api_version": "v1",
                "kind": "Node",
                "metadata": {
                    "annotations": {"node.alpha.kubernetes.io/ttl": "0"},
                    "labels": self.make_node_labels(name=name),
                    "name": name,
                    "namespace": None,
                    "self_link": "/api/v1/nodes/{name}".format(name=name),
                    "uid": "7811b8ae-c1a1-11e7-a55a-0800279fb61e",
                },
                "spec": {"external_id": name},
                "status": {},
            }
        )
        return node_data

    def make_namespace(self, name="default"):
        namespace_data = self.make_ret_dict(kind="Namespace", name=name)
        del namespace_data["data"]
        namespace_data.update(
            {
                "status": {"phase": "Active"},
                "spec": {"finalizers": ["kubernetes"]},
                "metadata": {
                    "name": name,
                    "namespace": None,
                    "labels": None,
                    "self_link": "/api/v1/namespaces/{namespace}".format(
                        namespace=name,
                    ),
                    "annotations": None,
                    "uid": "752fceeb-c1a1-11e7-a55a-0800279fb61e",
                },
            }
        )
        return namespace_data

    def make_ret_dict(self, kind, name, namespace=None, data=None):
        """
        Make a minimal example configmap or secret for using in mocks
        """

        assert kind in ("Secret", "ConfigMap", "Namespace", "Node")

        if data is None:
            data = {}

        self_link = "/api/v1/namespaces/{namespace}/{kind}s/{name}".format(
            namespace=namespace, kind=kind.lower(), name=name,
        )

        return_data = {
            "kind": kind,
            "data": data,
            "api_version": "v1",
            "metadata": {
                "name": name,
                "labels": None,
                "namespace": namespace,
                "self_link": self_link,
                "annotations": {"kubernetes.io/change-cause": "salt-call state.apply"},
            },
        }
        return return_data

    def test_configmap_present__fail(self):
        error = kubernetes.configmap_present(
            name="testme", data={1: 1}, source="salt://beyond/oblivion.jinja",
        )
        self.assertDictEqual(
            {
                "changes": {},
                "result": False,
                "name": "testme",
                "comment": "'source' cannot be used in combination with 'data'",
            },
            error,
        )

    def test_configmap_present__create_test_true(self):
        # Create a new configmap with test=True
        with self.mock_func("show_configmap", return_value=None, test=True):
            ret = kubernetes.configmap_present(
                name="example", data={"example.conf": "# empty config file"},
            )
            self.assertDictEqual(
                {
                    "comment": "The configmap is going to be created",
                    "changes": {},
                    "name": "example",
                    "result": None,
                },
                ret,
            )

    def test_configmap_present__create(self):
        # Create a new configmap
        with self.mock_func("show_configmap", return_value=None):
            cm = self.make_configmap(
                name="test", namespace="default", data={"foo": "bar"},
            )
            with self.mock_func("create_configmap", return_value=cm):
                actual = kubernetes.configmap_present(name="test", data={"foo": "bar"},)
                self.assertDictEqual(
                    {
                        "comment": "",
                        "changes": {"data": {"foo": "bar"}},
                        "name": "test",
                        "result": True,
                    },
                    actual,
                )

    def test_configmap_present__create_no_data(self):
        # Create a new configmap with no 'data' attribute
        with self.mock_func("show_configmap", return_value=None):
            cm = self.make_configmap(name="test", namespace="default",)
            with self.mock_func("create_configmap", return_value=cm):
                actual = kubernetes.configmap_present(name="test")
                self.assertDictEqual(
                    {
                        "comment": "",
                        "changes": {"data": {}},
                        "name": "test",
                        "result": True,
                    },
                    actual,
                )

    def test_configmap_present__replace_test_true(self):
        cm = self.make_configmap(
            name="settings",
            namespace="saltstack",
            data={"foobar.conf": "# Example configuration"},
        )
        with self.mock_func("show_configmap", return_value=cm, test=True):
            ret = kubernetes.configmap_present(
                name="settings",
                namespace="saltstack",
                data={"foobar.conf": "# Example configuration"},
            )
            self.assertDictEqual(
                {
                    "comment": "The configmap is going to be replaced",
                    "changes": {},
                    "name": "settings",
                    "result": None,
                },
                ret,
            )

    def test_configmap_present__replace(self):
        cm = self.make_configmap(name="settings", data={"action": "make=war"})
        # Replace an existing configmap
        with self.mock_func("show_configmap", return_value=cm):
            new_cm = cm.copy()
            new_cm.update({"data": {"action": "make=peace"}})
            with self.mock_func("replace_configmap", return_value=new_cm):
                actual = kubernetes.configmap_present(
                    name="settings", data={"action": "make=peace"},
                )
                self.assertDictEqual(
                    {
                        "comment": "The configmap is already present. Forcing recreation",
                        "changes": {"data": {"action": "make=peace"}},
                        "name": "settings",
                        "result": True,
                    },
                    actual,
                )

    def test_configmap_absent__noop_test_true(self):
        # Nothing to delete with test=True
        with self.mock_func("show_configmap", return_value=None, test=True):
            actual = kubernetes.configmap_absent(name="NOT_FOUND")
            self.assertDictEqual(
                {
                    "comment": "The configmap does not exist",
                    "changes": {},
                    "name": "NOT_FOUND",
                    "result": None,
                },
                actual,
            )

    def test_configmap_absent__test_true(self):
        # Configmap exists with test=True
        cm = self.make_configmap(name="deleteme", namespace="default")
        with self.mock_func("show_configmap", return_value=cm, test=True):
            actual = kubernetes.configmap_absent(name="deleteme")
            self.assertDictEqual(
                {
                    "comment": "The configmap is going to be deleted",
                    "changes": {},
                    "name": "deleteme",
                    "result": None,
                },
                actual,
            )

    def test_configmap_absent__noop(self):
        # Nothing to delete
        with self.mock_func("show_configmap", return_value=None):
            actual = kubernetes.configmap_absent(name="NOT_FOUND")
            self.assertDictEqual(
                {
                    "comment": "The configmap does not exist",
                    "changes": {},
                    "name": "NOT_FOUND",
                    "result": True,
                },
                actual,
            )

    def test_configmap_absent(self):
        # Configmap exists, delete it!
        cm = self.make_configmap(name="deleteme", namespace="default")
        with self.mock_func("show_configmap", return_value=cm):
            # The return from this module isn't used in the state
            with self.mock_func("delete_configmap", return_value={}):
                actual = kubernetes.configmap_absent(name="deleteme")
                self.assertDictEqual(
                    {
                        "comment": "ConfigMap deleted",
                        "changes": {
                            "kubernetes.configmap": {
                                "new": "absent",
                                "old": "present",
                            },
                        },
                        "name": "deleteme",
                        "result": True,
                    },
                    actual,
                )

    def test_secret_present__fail(self):
        actual = kubernetes.secret_present(
            name="sekret", data={"password": "monk3y"}, source="salt://nope.jinja",
        )
        self.assertDictEqual(
            {
                "changes": {},
                "result": False,
                "name": "sekret",
                "comment": "'source' cannot be used in combination with 'data'",
            },
            actual,
        )

    def test_secret_present__exists_test_true(self):
        secret = self.make_secret(name="sekret")
        new_secret = secret.copy()
        new_secret.update({"data": {"password": "uncle"}})
        # Secret exists already and needs replacing with test=True
        with self.mock_func("show_secret", return_value=secret):
            with self.mock_func("replace_secret", return_value=new_secret, test=True):
                actual = kubernetes.secret_present(
                    name="sekret", data={"password": "uncle"},
                )
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": None,
                        "name": "sekret",
                        "comment": "The secret is going to be replaced",
                    },
                    actual,
                )

    def test_secret_present__exists(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name="sekret", data={"password": "booyah"})
        with self.mock_func("show_secret", return_value=secret):
            with self.mock_func("replace_secret", return_value=secret):
                actual = kubernetes.secret_present(
                    name="sekret", data={"password": "booyah"},
                )
                self.assertDictEqual(
                    {
                        "changes": {"data": ["password"]},
                        "result": True,
                        "name": "sekret",
                        "comment": "The secret is already present. Forcing recreation",
                    },
                    actual,
                )

    def test_secret_present__create(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name="sekret", data={"password": "booyah"})
        with self.mock_func("show_secret", return_value=None):
            with self.mock_func("create_secret", return_value=secret):
                actual = kubernetes.secret_present(
                    name="sekret", data={"password": "booyah"},
                )
                self.assertDictEqual(
                    {
                        "changes": {"data": ["password"]},
                        "result": True,
                        "name": "sekret",
                        "comment": "",
                    },
                    actual,
                )

    def test_secret_present__create_no_data(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name="sekret")
        with self.mock_func("show_secret", return_value=None):
            with self.mock_func("create_secret", return_value=secret):
                actual = kubernetes.secret_present(name="sekret")
                self.assertDictEqual(
                    {
                        "changes": {"data": []},
                        "result": True,
                        "name": "sekret",
                        "comment": "",
                    },
                    actual,
                )

    def test_secret_present__create_test_true(self):
        # Secret exists and gets replaced with test=True
        secret = self.make_secret(name="sekret")
        with self.mock_func("show_secret", return_value=None):
            with self.mock_func("create_secret", return_value=secret, test=True):
                actual = kubernetes.secret_present(name="sekret")
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": None,
                        "name": "sekret",
                        "comment": "The secret is going to be created",
                    },
                    actual,
                )

    def test_secret_absent__noop_test_true(self):
        with self.mock_func("show_secret", return_value=None, test=True):
            actual = kubernetes.secret_absent(name="sekret")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "sekret",
                    "comment": "The secret does not exist",
                },
                actual,
            )

    def test_secret_absent__noop(self):
        with self.mock_func("show_secret", return_value=None):
            actual = kubernetes.secret_absent(name="passwords")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": True,
                    "name": "passwords",
                    "comment": "The secret does not exist",
                },
                actual,
            )

    def test_secret_absent__delete_test_true(self):
        secret = self.make_secret(name="credentials", data={"redis": "letmein"})
        with self.mock_func("show_secret", return_value=secret):
            with self.mock_func("delete_secret", return_value=secret, test=True):
                actual = kubernetes.secret_absent(name="credentials")
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": None,
                        "name": "credentials",
                        "comment": "The secret is going to be deleted",
                    },
                    actual,
                )

    def test_secret_absent__delete(self):
        secret = self.make_secret(name="foobar", data={"redis": "letmein"})
        deleted = {
            "status": None,
            "kind": "Secret",
            "code": None,
            "reason": None,
            "details": None,
            "message": None,
            "api_version": "v1",
            "metadata": {
                "self_link": "/api/v1/namespaces/default/secrets/foobar",
                "resource_version": "30292",
            },
        }
        with self.mock_func("show_secret", return_value=secret):
            with self.mock_func("delete_secret", return_value=deleted):
                actual = kubernetes.secret_absent(name="foobar")
                self.assertDictEqual(
                    {
                        "changes": {
                            "kubernetes.secret": {"new": "absent", "old": "present"},
                        },
                        "result": True,
                        "name": "foobar",
                        "comment": "Secret deleted",
                    },
                    actual,
                )

    def test_node_label_present__add_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func("node_labels", return_value=labels, test=True):
            actual = kubernetes.node_label_present(
                name="com.zoo-animal", node="minikube", value="monkey",
            )
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "com.zoo-animal",
                    "comment": "The label is going to be set",
                },
                actual,
            )

    def test_node_label_present__add(self):
        node_data = self.make_node()
        # Remove some of the defaults to make it simpler
        node_data["metadata"]["labels"] = {
            "beta.kubernetes.io/os": "linux",
        }
        labels = node_data["metadata"]["labels"]

        with self.mock_func("node_labels", return_value=labels):
            with self.mock_func("node_add_label", return_value=node_data):
                actual = kubernetes.node_label_present(
                    name="failure-domain.beta.kubernetes.io/zone",
                    node="minikube",
                    value="us-central1-a",
                )
                self.assertDictEqual(
                    {
                        "comment": "",
                        "changes": {
                            "minikube.failure-domain.beta.kubernetes.io/zone": {
                                "new": {
                                    "failure-domain.beta.kubernetes.io/zone": "us-central1-a",
                                    "beta.kubernetes.io/os": "linux",
                                },
                                "old": {"beta.kubernetes.io/os": "linux"},
                            },
                        },
                        "name": "failure-domain.beta.kubernetes.io/zone",
                        "result": True,
                    },
                    actual,
                )

    def test_node_label_present__already_set(self):
        node_data = self.make_node()
        labels = node_data["metadata"]["labels"]
        with self.mock_func("node_labels", return_value=labels):
            with self.mock_func("node_add_label", return_value=node_data):
                actual = kubernetes.node_label_present(
                    name="failure-domain.beta.kubernetes.io/region",
                    node="minikube",
                    value="us-west-1",
                )
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": True,
                        "name": "failure-domain.beta.kubernetes.io/region",
                        "comment": "The label is already set and has the specified value",
                    },
                    actual,
                )

    def test_node_label_present__update_test_true(self):
        node_data = self.make_node()
        labels = node_data["metadata"]["labels"]
        with self.mock_func("node_labels", return_value=labels):
            with self.mock_func("node_add_label", return_value=node_data, test=True):
                actual = kubernetes.node_label_present(
                    name="failure-domain.beta.kubernetes.io/region",
                    node="minikube",
                    value="us-east-1",
                )
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": None,
                        "name": "failure-domain.beta.kubernetes.io/region",
                        "comment": "The label is going to be updated",
                    },
                    actual,
                )

    def test_node_label_present__update(self):
        node_data = self.make_node()
        # Remove some of the defaults to make it simpler
        node_data["metadata"]["labels"] = {
            "failure-domain.beta.kubernetes.io/region": "us-west-1",
        }
        labels = node_data["metadata"]["labels"]
        with self.mock_func("node_labels", return_value=labels):
            with self.mock_func("node_add_label", return_value=node_data):
                actual = kubernetes.node_label_present(
                    name="failure-domain.beta.kubernetes.io/region",
                    node="minikube",
                    value="us-east-1",
                )
                self.assertDictEqual(
                    {
                        "changes": {
                            "minikube.failure-domain.beta.kubernetes.io/region": {
                                "new": {
                                    "failure-domain.beta.kubernetes.io/region": "us-east-1"
                                },
                                "old": {
                                    "failure-domain.beta.kubernetes.io/region": "us-west-1"
                                },
                            }
                        },
                        "result": True,
                        "name": "failure-domain.beta.kubernetes.io/region",
                        "comment": "The label is already set, changing the value",
                    },
                    actual,
                )

    def test_node_label_absent__noop_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func("node_labels", return_value=labels, test=True):
            actual = kubernetes.node_label_absent(
                name="non-existent-label", node="minikube",
            )
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "non-existent-label",
                    "comment": "The label does not exist",
                },
                actual,
            )

    def test_node_label_absent__noop(self):
        labels = self.make_node_labels()
        with self.mock_func("node_labels", return_value=labels):
            actual = kubernetes.node_label_absent(
                name="non-existent-label", node="minikube",
            )
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": True,
                    "name": "non-existent-label",
                    "comment": "The label does not exist",
                },
                actual,
            )

    def test_node_label_absent__delete_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func("node_labels", return_value=labels, test=True):
            actual = kubernetes.node_label_absent(
                name="failure-domain.beta.kubernetes.io/region", node="minikube",
            )
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "failure-domain.beta.kubernetes.io/region",
                    "comment": "The label is going to be deleted",
                },
                actual,
            )

    def test_node_label_absent__delete(self):
        node_data = self.make_node()
        labels = node_data["metadata"]["labels"].copy()

        node_data["metadata"]["labels"].pop("failure-domain.beta.kubernetes.io/region")

        with self.mock_func("node_labels", return_value=labels):
            with self.mock_func("node_remove_label", return_value=node_data):
                actual = kubernetes.node_label_absent(
                    name="failure-domain.beta.kubernetes.io/region", node="minikube",
                )
                self.assertDictEqual(
                    {
                        "result": True,
                        "changes": {
                            "kubernetes.node_label": {
                                "new": "absent",
                                "old": "present",
                            }
                        },
                        "comment": "Label removed from node",
                        "name": "failure-domain.beta.kubernetes.io/region",
                    },
                    actual,
                )

    def test_namespace_present__create_test_true(self):
        with self.mock_func("show_namespace", return_value=None, test=True):
            actual = kubernetes.namespace_present(name="saltstack")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "saltstack",
                    "comment": "The namespace is going to be created",
                },
                actual,
            )

    def test_namespace_present__create(self):
        namespace_data = self.make_namespace(name="saltstack")
        with self.mock_func("show_namespace", return_value=None):
            with self.mock_func("create_namespace", return_value=namespace_data):
                actual = kubernetes.namespace_present(name="saltstack")
                self.assertDictEqual(
                    {
                        "changes": {"namespace": {"new": namespace_data, "old": {}}},
                        "result": True,
                        "name": "saltstack",
                        "comment": "",
                    },
                    actual,
                )

    def test_namespace_present__noop_test_true(self):
        namespace_data = self.make_namespace(name="saltstack")
        with self.mock_func("show_namespace", return_value=namespace_data, test=True):
            actual = kubernetes.namespace_present(name="saltstack")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "saltstack",
                    "comment": "The namespace already exists",
                },
                actual,
            )

    def test_namespace_present__noop(self):
        namespace_data = self.make_namespace(name="saltstack")
        with self.mock_func("show_namespace", return_value=namespace_data):
            actual = kubernetes.namespace_present(name="saltstack")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": True,
                    "name": "saltstack",
                    "comment": "The namespace already exists",
                },
                actual,
            )

    def test_namespace_absent__noop_test_true(self):
        with self.mock_func("show_namespace", return_value=None, test=True):
            actual = kubernetes.namespace_absent(name="salt")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "salt",
                    "comment": "The namespace does not exist",
                },
                actual,
            )

    def test_namespace_absent__noop(self):
        with self.mock_func("show_namespace", return_value=None):
            actual = kubernetes.namespace_absent(name="salt")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": True,
                    "name": "salt",
                    "comment": "The namespace does not exist",
                },
                actual,
            )

    def test_namespace_absent__delete_test_true(self):
        namespace_data = self.make_namespace(name="salt")
        with self.mock_func("show_namespace", return_value=namespace_data, test=True):
            actual = kubernetes.namespace_absent(name="salt")
            self.assertDictEqual(
                {
                    "changes": {},
                    "result": None,
                    "name": "salt",
                    "comment": "The namespace is going to be deleted",
                },
                actual,
            )

    def test_namespace_absent__delete_code_200(self):
        namespace_data = self.make_namespace(name="salt")
        deleted = namespace_data.copy()
        deleted["code"] = 200
        deleted.update({"code": 200, "message": None})
        with self.mock_func("show_namespace", return_value=namespace_data):
            with self.mock_func("delete_namespace", return_value=deleted):
                actual = kubernetes.namespace_absent(name="salt")
                self.assertDictEqual(
                    {
                        "changes": {
                            "kubernetes.namespace": {"new": "absent", "old": "present"}
                        },
                        "result": True,
                        "name": "salt",
                        "comment": "Terminating",
                    },
                    actual,
                )

    def test_namespace_absent__delete_status_terminating(self):
        namespace_data = self.make_namespace(name="salt")
        deleted = namespace_data.copy()
        deleted.update(
            {
                "code": None,
                "status": "Terminating namespace",
                "message": "Terminating this shizzzle yo",
            }
        )
        with self.mock_func("show_namespace", return_value=namespace_data):
            with self.mock_func("delete_namespace", return_value=deleted):
                actual = kubernetes.namespace_absent(name="salt")
                self.assertDictEqual(
                    {
                        "changes": {
                            "kubernetes.namespace": {"new": "absent", "old": "present"}
                        },
                        "result": True,
                        "name": "salt",
                        "comment": "Terminating this shizzzle yo",
                    },
                    actual,
                )

    def test_namespace_absent__delete_status_phase_terminating(self):
        # This is what kubernetes 1.8.0 looks like when deleting namespaces
        namespace_data = self.make_namespace(name="salt")
        deleted = namespace_data.copy()
        deleted.update(
            {"code": None, "message": None, "status": {"phase": "Terminating"}}
        )
        with self.mock_func("show_namespace", return_value=namespace_data):
            with self.mock_func("delete_namespace", return_value=deleted):
                actual = kubernetes.namespace_absent(name="salt")
                self.assertDictEqual(
                    {
                        "changes": {
                            "kubernetes.namespace": {"new": "absent", "old": "present"}
                        },
                        "result": True,
                        "name": "salt",
                        "comment": "Terminating",
                    },
                    actual,
                )

    def test_namespace_absent__delete_error(self):
        namespace_data = self.make_namespace(name="salt")
        deleted = namespace_data.copy()
        deleted.update({"code": 418, "message": "I' a teapot!", "status": None})
        with self.mock_func("show_namespace", return_value=namespace_data):
            with self.mock_func("delete_namespace", return_value=deleted):
                actual = kubernetes.namespace_absent(name="salt")
                self.assertDictEqual(
                    {
                        "changes": {},
                        "result": False,
                        "name": "salt",
                        "comment": "Something went wrong, response: {0}".format(
                            deleted,
                        ),
                    },
                    actual,
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.locale as locale

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LocaleTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the locale state
    """

    def setup_loader_modules(self):
        return {locale: {}}

    def test_system(self):
        """
            Test to set the locale for the system
        """
        ret = [
            {
                "changes": {},
                "comment": "System locale salt already set",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "System locale saltstack needs to be set",
                "name": "saltstack",
                "result": None,
            },
            {
                "changes": {"locale": "saltstack"},
                "comment": "Set system locale saltstack",
                "name": "saltstack",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Failed to set system locale to saltstack",
                "name": "saltstack",
                "result": False,
            },
        ]

        mock = MagicMock(return_value="salt")
        with patch.dict(locale.__salt__, {"locale.get_locale": mock}):
            self.assertDictEqual(locale.system("salt"), ret[0])

            with patch.dict(locale.__opts__, {"test": True}):
                self.assertDictEqual(locale.system("saltstack"), ret[1])

            with patch.dict(locale.__opts__, {"test": False}):
                mock = MagicMock(side_effect=[True, False])
                with patch.dict(locale.__salt__, {"locale.set_locale": mock}):
                    self.assertDictEqual(locale.system("saltstack"), ret[2])

                    self.assertDictEqual(locale.system("saltstack"), ret[3])

    def test_present(self):
        """
            Test to generate a locale if it is not present
        """
        ret = [
            {
                "changes": {},
                "comment": "Locale salt is already present",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Locale salt needs to be generated",
                "name": "salt",
                "result": None,
            },
            {
                "changes": {"locale": "salt"},
                "comment": "Generated locale salt",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Failed to generate locale salt",
                "name": "salt",
                "result": False,
            },
        ]

        mock = MagicMock(side_effect=[True, False, False, False])
        with patch.dict(locale.__salt__, {"locale.avail": mock}):
            self.assertDictEqual(locale.present("salt"), ret[0])

            with patch.dict(locale.__opts__, {"test": True}):
                self.assertDictEqual(locale.present("salt"), ret[1])

            with patch.dict(locale.__opts__, {"test": False}):
                mock = MagicMock(side_effect=[True, False])
                with patch.dict(locale.__salt__, {"locale.gen_locale": mock}):
                    self.assertDictEqual(locale.present("salt"), ret[2])

                    self.assertDictEqual(locale.present("salt"), ret[3])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Justin Anderson <janderson@saltstack.com>
"""

from __future__ import absolute_import, print_function, unicode_literals

import os

import pytest
from salt.exceptions import CommandExecutionError
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf


@pytest.mark.windows_whitelisted
class BeaconsAddDeleteTest(ModuleCase):
    """
    Tests the add and delete functions
    """

    def setUp(self):
        self.minion_conf_d_dir = os.path.join(
            RUNTIME_VARS.TMP_CONF_DIR,
            os.path.dirname(self.minion_opts["default_include"]),
        )
        if not os.path.isdir(self.minion_conf_d_dir):
            os.makedirs(self.minion_conf_d_dir)
        self.beacons_config_file_path = os.path.join(
            self.minion_conf_d_dir, "beacons.conf"
        )
        self.run_function("beacons.reset", f_timeout=300)

    def tearDown(self):
        if os.path.isfile(self.beacons_config_file_path):
            os.unlink(self.beacons_config_file_path)

        # Reset beacons
        self.run_function("beacons.reset", f_timeout=300)

    @slowTest
    def test_add_and_delete(self):
        """
        Test adding and deleting a beacon
        """
        _add = self.run_function(
            "beacons.add",
            ["ps", [{"processes": {"apache2": "stopped"}}]],
            f_timeout=300,
        )
        self.assertTrue(_add["result"])

        # save added beacon
        _save = self.run_function("beacons.save", f_timeout=300)
        self.assertTrue(_save["result"])

        # delete the beacon
        _delete = self.run_function("beacons.delete", ["ps"], f_timeout=300)
        self.assertTrue(_delete["result"])

        # save the results
        self.run_function("beacons.save", f_timeout=300)

    @slowTest
    def test_add_and_delete_beacon_module(self):
        """
        Test adding and deleting a beacon
        """
        _add = self.run_function(
            "beacons.add",
            [
                "watch_apache",
                [{"processes": {"apache2": "stopped"}}, {"beacon_module": "ps"}],
            ],
            f_timeout=300,
        )
        self.assertTrue(_add["result"])

        # save added beacon
        _save = self.run_function("beacons.save", f_timeout=300)
        self.assertTrue(_save["result"])

        # delete the beacon
        _delete = self.run_function("beacons.delete", ["ps"], f_timeout=300)
        self.assertTrue(_delete["result"])

        # save the results
        self.run_function("beacons.save", f_timeout=300)


@pytest.mark.windows_whitelisted
class BeaconsTest(ModuleCase):
    """
    Tests the beacons execution module
    """

    beacons_config_file_path = minion_conf_d_dir = None

    @classmethod
    def tearDownClass(cls):
        if cls.beacons_config_file_path and os.path.isfile(
            cls.beacons_config_file_path
        ):
            os.unlink(cls.beacons_config_file_path)

    def setUp(self):
        if self.minion_conf_d_dir is None:
            self.minion_conf_d_dir = os.path.join(
                RUNTIME_VARS.TMP_CONF_DIR,
                os.path.dirname(self.minion_opts["default_include"]),
            )
            if not os.path.isdir(self.minion_conf_d_dir):
                os.makedirs(self.minion_conf_d_dir)
        self.__class__.beacons_config_file_path = os.path.join(
            self.minion_conf_d_dir, "beacons.conf"
        )
        self.run_function("beacons.reset", f_timeout=300)
        try:
            # Add beacon to disable
            self.run_function(
                "beacons.add",
                ["ps", [{"processes": {"apache2": "stopped"}}]],
                f_timeout=300,
            )
            self.run_function("beacons.save", f_timeout=300)
        except CommandExecutionError:
            self.skipTest("Unable to add beacon")

    def tearDown(self):
        # delete added beacon
        self.run_function("beacons.delete", ["ps"], f_timeout=300)
        self.run_function("beacons.save", f_timeout=300)

        # Reset beacons
        self.run_function("beacons.reset", f_timeout=300)

    @slowTest
    def test_disable(self):
        """
        Test disabling beacons
        """
        # assert beacon exists
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertIn("ps", _list)

        ret = self.run_function("beacons.disable", f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacons are disabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertFalse(_list["enabled"])

        # disable added beacon
        ret = self.run_function("beacons.disable_beacon", ["ps"], f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacon ps is disabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        for bdict in _list["ps"]:
            if "enabled" in bdict:
                self.assertFalse(bdict["enabled"])
                break

    @slowTest
    def test_enable(self):
        """
        Test enabling beacons
        """
        # assert beacon exists
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertIn("ps", _list)

        # enable beacons on minion
        ret = self.run_function("beacons.enable", f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacons are enabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertTrue(_list["enabled"])

    @skipIf(
        True,
        "Skip until https://github.com/saltstack/salt/issues/31516 "
        "problems are resolved.",
    )
    def test_enabled_beacons(self):
        """
        Test enabled specific beacon
        """
        # enable added beacon
        ret = self.run_function("beacons.enable_beacon", ["ps"], f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacon ps is enabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertTrue(_list["ps"]["enabled"])

    @slowTest
    def test_list(self):
        """
        Test listing the beacons
        """
        # list beacons
        ret = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        if "enabled" in ret:
            self.assertEqual(
                ret, {"ps": [{"processes": {"apache2": "stopped"}}], "enabled": True}
            )
        else:
            self.assertEqual(ret, {"ps": [{"processes": {"apache2": "stopped"}}]})

    @slowTest
    def test_list_available(self):
        """
        Test listing the beacons
        """
        # list beacons
        ret = self.run_function(
            "beacons.list_available", return_yaml=False, f_timeout=300
        )
        self.assertTrue(ret)


class BeaconsWithBeaconTypeTest(ModuleCase):
    """
    Tests the beacons execution module
    """

    beacons_config_file_path = minion_conf_d_dir = None

    @classmethod
    def tearDownClass(cls):
        if cls.beacons_config_file_path and os.path.isfile(
            cls.beacons_config_file_path
        ):
            os.unlink(cls.beacons_config_file_path)

    def setUp(self):
        if self.minion_conf_d_dir is None:
            self.minion_conf_d_dir = os.path.join(
                RUNTIME_VARS.TMP_CONF_DIR,
                os.path.dirname(self.minion_opts["default_include"]),
            )
            if not os.path.isdir(self.minion_conf_d_dir):
                os.makedirs(self.minion_conf_d_dir)
        self.__class__.beacons_config_file_path = os.path.join(
            self.minion_conf_d_dir, "beacons.conf"
        )
        self.run_function("beacons.reset", f_timeout=300)
        try:
            # Add beacon to disable
            self.run_function(
                "beacons.add",
                [
                    "watch_apache",
                    [{"processes": {"apache2": "stopped"}}, {"beacon_module": "ps"}],
                ],
                f_timeout=300,
            )
            self.run_function("beacons.save", f_timeout=300)
        except CommandExecutionError:
            self.skipTest("Unable to add beacon")

    def tearDown(self):
        # delete added beacon
        self.run_function("beacons.delete", ["watch_apache"], f_timeout=300)
        self.run_function("beacons.save", f_timeout=300)

    @slowTest
    def test_disable(self):
        """
        Test disabling beacons
        """
        ret = self.run_function("beacons.enable", f_timeout=300)
        self.assertTrue(ret["result"])
        # assert beacon exists
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertIn("watch_apache", _list)

        ret = self.run_function("beacons.disable", f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacons are disabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertFalse(_list["enabled"])

        # disable added beacon
        ret = self.run_function(
            "beacons.disable_beacon", ["watch_apache"], f_timeout=300
        )
        self.assertTrue(ret["result"])

        # assert beacon ps is disabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        for bdict in _list["watch_apache"]:
            if "enabled" in bdict:
                self.assertFalse(bdict["enabled"])
                break

    @slowTest
    def test_enable(self):
        """
        Test enabling beacons
        """
        # assert beacon exists
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertIn("watch_apache", _list)

        # enable beacons on minion
        ret = self.run_function("beacons.enable", f_timeout=300)
        self.assertTrue(ret["result"])

        # assert beacons are enabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertTrue(_list["enabled"])

    @skipIf(
        True,
        "Skip until https://github.com/saltstack/salt/issues/31516 problems are resolved.",
    )
    def test_enabled_beacons(self):
        """
        Test enabled specific beacon
        """
        # enable added beacon
        ret = self.run_function(
            "beacons.enable_beacon", ["watch_apache"], f_timeout=300
        )
        self.assertTrue(ret["result"])

        # assert beacon ps is enabled
        _list = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        self.assertTrue(_list["watch_apache"]["enabled"])

    @slowTest
    def test_list(self):
        """
        Test listing the beacons
        """
        # list beacons
        ret = self.run_function("beacons.list", return_yaml=False, f_timeout=300)
        _expected = {
            "watch_apache": [
                {"processes": {"apache2": "stopped"}},
                {"beacon_module": "ps"},
            ]
        }
        _enabled_expected = {
            "watch_apache": [
                {"processes": {"apache2": "stopped"}},
                {"beacon_module": "ps"},
            ],
            "enabled": True,
        }
        if "enabled" in ret:
            self.assertEqual(ret, _enabled_expected)
        else:
            self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import pytest
from tests.support.case import SyndicCase
from tests.support.helpers import slowTest


@pytest.mark.windows_whitelisted
class TestSyndic(SyndicCase):
    """
    Validate the syndic interface by testing the test module
    """

    @slowTest
    def test_ping(self):
        """
        test.ping
        """
        self.assertTrue(self.run_function("test.ping"))

    @slowTest
    def test_fib(self):
        """
        test.fib
        """
        self.assertEqual(self.run_function("test.fib", ["20"],)[0], 6765)



# -*- coding: utf-8 -*-
"""
Integration tests for the lxd states
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Lxd Test Case
import tests.integration.states.test_lxd


class LxdImageTestCase(tests.integration.states.test_lxd.LxdTestCase):
    def test_02__pull_image(self):
        ret = self.run_state(
            "lxd_image.present",
            name="images:centos/7",
            source={
                "name": "centos/7",
                "type": "simplestreams",
                "server": "https://images.linuxcontainers.org",
            },
        )
        name = "lxd_image_|-images:centos/7_|-images:centos/7_|-present"
        self.assertSaltTrueReturn(ret)
        assert name in ret
        assert ret[name]["changes"]["aliases"] == ['Added alias "images:centos/7"']

    def test_03__delete_image(self):
        ret = self.run_state("lxd_image.absent", name="images:centos/7",)
        name = "lxd_image_|-images:centos/7_|-images:centos/7_|-absent"
        self.assertSaltTrueReturn(ret)
        assert name in ret
        assert (
            ret[name]["changes"]["removed"]
            == 'Image "images:centos/7" has been deleted.'
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import pytest
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest


@pytest.mark.windows_whitelisted
class AliasesTest(ModuleCase):
    """
    Validate aliases module
    """

    @slowTest
    def test_set_target(self):
        """
        aliases.set_target and aliases.get_target
        """
        set_ret = self.run_function("aliases.set_target", alias="fred", target="bob")
        self.assertTrue(set_ret)
        tgt_ret = self.run_function("aliases.get_target", alias="fred")
        self.assertEqual(tgt_ret, "bob")

    @slowTest
    def test_has_target(self):
        """
        aliases.set_target and aliases.has_target
        """
        set_ret = self.run_function("aliases.set_target", alias="fred", target="bob")
        self.assertTrue(set_ret)
        tgt_ret = self.run_function("aliases.has_target", alias="fred", target="bob")
        self.assertTrue(tgt_ret)

    @slowTest
    def test_list_aliases(self):
        """
        aliases.list_aliases
        """
        set_ret = self.run_function("aliases.set_target", alias="fred", target="bob")
        self.assertTrue(set_ret)
        tgt_ret = self.run_function("aliases.list_aliases")
        self.assertIsInstance(tgt_ret, dict)
        self.assertIn("fred", tgt_ret)

    @slowTest
    def test_rm_alias(self):
        """
        aliases.rm_alias
        """
        set_ret = self.run_function("aliases.set_target", alias="frank", target="greg")
        self.assertTrue(set_ret)
        self.run_function("aliases.rm_alias", alias="frank")
        tgt_ret = self.run_function("aliases.list_aliases")
        self.assertIsInstance(tgt_ret, dict)
        self.assertNotIn("alias=frank", tgt_ret)



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.openvswitch_port as openvswitch_port

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class OpenvswitchPortTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.openvswitch_port
    """

    def setup_loader_modules(self):
        return {openvswitch_port: {"__opts__": {"test": False}}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verify that the named port exists on bridge, eventually creates it.
        """
        name = "salt"
        bridge = "br-salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(return_value=True)
        mock_l = MagicMock(return_value=["salt"])
        mock_n = MagicMock(return_value=[])

        with patch.dict(
            openvswitch_port.__salt__,
            {
                "openvswitch.bridge_exists": mock,
                "openvswitch.interface_get_type": MagicMock(return_value='""'),
                "openvswitch.port_list": mock_l,
            },
        ):
            comt = "Port salt already exists."
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(openvswitch_port.present(name, bridge), ret)

        with patch.dict(
            openvswitch_port.__salt__,
            {
                "openvswitch.bridge_exists": mock,
                "openvswitch.interface_get_type": MagicMock(return_value='""'),
                "openvswitch.port_list": mock_n,
                "openvswitch.port_add": mock,
            },
        ):
            comt = "Port salt created on bridge br-salt."
            ret.update(
                {
                    "comment": comt,
                    "result": True,
                    "changes": {
                        "salt": {
                            "new": "Created port salt on bridge br-salt.",
                            "old": "No port named salt present.",
                        },
                    },
                }
            )
            self.assertDictEqual(openvswitch_port.present(name, bridge), ret)
        with patch.dict(
            openvswitch_port.__salt__,
            {
                "openvswitch.bridge_exists": mock,
                "openvswitch.port_list": mock_n,
                "openvswitch.port_add": mock,
                "openvswitch.interface_get_options": mock_n,
                "openvswitch.interface_get_type": MagicMock(return_value=""),
                "openvswitch.port_create_gre": mock,
                "dig.check_ip": mock,
            },
        ):
            comt = "Port salt created on bridge br-salt."
            self.maxDiff = None
            ret.update(
                {
                    "result": True,
                    "comment": "Created GRE tunnel interface salt with remote ip 10.0.0.1  and key 1 on bridge br-salt.",
                    "changes": {
                        "salt": {
                            "new": "Created GRE tunnel interface salt with remote ip 10.0.0.1 and key 1 on bridge br-salt.",
                            "old": "No GRE tunnel interface salt with remote ip 10.0.0.1 and key 1 on bridge br-salt present.",
                        },
                    },
                }
            )
            self.assertDictEqual(
                openvswitch_port.present(
                    name, bridge, tunnel_type="gre", id=1, remote="10.0.0.1"
                ),
                ret,
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.nexus as nexus

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class nexusTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.nexus
    """

    def setup_loader_modules(self):
        return {nexus: {}}

    # 'downloaded' function tests: 1

    def test_downloaded(self):
        """
        Test to ensures that the artifact from nexus exists at
        given location.
        """
        name = "jboss"
        arti_url = "http://nexus.example.com/repository"
        artifact = {
            "nexus_url": arti_url,
            "artifact_id": "module",
            "repository": "libs-releases",
            "packaging": "jar",
            "group_id": "com.company.module",
            "classifier": "sources",
            "version": "1.0",
        }

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        mck = MagicMock(return_value={"status": False, "changes": {}, "comment": ""})
        with patch.dict(nexus.__salt__, {"nexus.get_release": mck}):
            self.assertDictEqual(nexus.downloaded(name, artifact), ret)

        with patch.object(
            nexus, "__fetch_from_nexus", MagicMock(side_effect=Exception("error"))
        ):
            ret = nexus.downloaded(name, artifact)
            self.assertEqual(ret["result"], False)
            self.assertEqual(ret["comment"], "error")



# -*- coding: utf-8 -*-
"""
Tests for the rabbitmq state
"""
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import skip_if_not_root
from tests.support.mixins import SaltReturnAssertsMixin


@skip_if_not_root
class RabbitUserTestCase(ModuleCase, SaltReturnAssertsMixin):
    """
    Validate the rabbitmq user states.
    """

    def setUp(self):
        super(RabbitUserTestCase, self).setUp()
        rabbit_installed = self.run_function("cmd.has_exec", ["rabbitmqctl"])

        if not rabbit_installed:
            self.skipTest("rabbitmq-server not installed")

    def test_present(self):
        """
        rabbitmq_user.present null_name
        """
        ret = self.run_state("rabbitmq_user.present", name="null_name", test=True)
        self.assertSaltFalseReturn(ret)
        self.assertInSaltComment("User 'null_name' is set to be created", ret)

    def absent(self):
        """
        rabbitmq_user.absent null_name
        """
        ret = self.run_state("rabbitmq_user.absent", name="null_name", test=True)
        self.assertSaltFalseReturn(ret)



# -*- coding: utf-8 -*-
"""
Tests for the Azure Blob External Pillar.
"""
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import os
import pickle
import tempfile
import time

import salt.config
import salt.loader

# Import Salt Libs
import salt.pillar.azureblob as azureblob
import salt.utils.files
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf

# Import Azure libs
HAS_LIBS = False
try:
    # pylint: disable=no-name-in-module
    from azure.storage.blob import BlobServiceClient

    # pylint: enable=no-name-in-module

    HAS_LIBS = True
except ImportError:
    pass


class MockBlob(dict):
    """
    Creates a Mock Blob object.
    """

    name = ""

    def __init__(self):
        super(MockBlob, self).__init__(
            {
                "container": None,
                "name": "test.sls",
                "prefix": None,
                "delimiter": "/",
                "results_per_page": None,
                "location_mode": None,
            }
        )


class MockContainerClient(object):
    """
    Creates a Mock ContainerClient.
    """

    def __init__(self):
        pass

    def walk_blobs(self, *args, **kwargs):
        yield MockBlob()

    def get_blob_client(self, *args, **kwargs):
        pass


class MockBlobServiceClient(object):
    """
    Creates a Mock BlobServiceClient.
    """

    def __init__(self):
        pass

    def get_container_client(self, *args, **kwargs):
        container_client = MockContainerClient()
        return container_client


@skipIf(HAS_LIBS is False, "The azure.storage.blob module must be installed.")
class AzureBlobTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.pillar.azureblob ext_pillar.
    """

    def setup_loader_modules(self):
        self.opts = salt.config.DEFAULT_MASTER_OPTS.copy()
        utils = salt.loader.utils(self.opts)
        return {
            azureblob: {"__opts__": self.opts, "__utils__": utils},
        }

    def test__init_expired(self):
        """
        Tests the result of _init when the cache is expired.
        """
        container = "test"
        multiple_env = False
        environment = "base"
        blob_cache_expire = 0  # The cache will be expired
        blob_client = MockBlobServiceClient()
        cache_file = tempfile.NamedTemporaryFile()
        # Patches the _get_containers_cache_filename module so that it returns the name of the new tempfile that
        # represents the cache file
        with patch.object(
            azureblob,
            "_get_containers_cache_filename",
            MagicMock(return_value=str(cache_file.name)),
        ):
            # Patches the from_connection_string module of the BlobServiceClient class so that a connection string does
            # not need to be given. Additionally it returns example blob data used by the ext_pillar.
            with patch.object(
                BlobServiceClient,
                "from_connection_string",
                MagicMock(return_value=blob_client),
            ):
                ret = azureblob._init(
                    "", container, multiple_env, environment, blob_cache_expire
                )
        cache_file.close()
        self.assertEqual(
            ret,
            {
                "base": {
                    "test": [
                        {
                            "container": None,
                            "name": "test.sls",
                            "prefix": None,
                            "delimiter": "/",
                            "results_per_page": None,
                            "location_mode": None,
                        }
                    ]
                }
            },
        )

    def test__init_not_expired(self):
        """
        Tests the result of _init when the cache is not expired.
        """
        container = "test"
        multiple_env = False
        environment = "base"
        blob_cache_expire = (time.time()) * (
            time.time()
        )  # The cache will not be expired
        metadata = {
            "base": {
                "test": [
                    {"name": "base/secret.sls", "relevant": "include.sls"},
                    {"name": "blobtest.sls", "irrelevant": "ignore.sls"},
                ]
            }
        }
        cache_file = tempfile.NamedTemporaryFile()
        # Pickles the metadata and stores it in cache_file
        with salt.utils.files.fopen(str(cache_file), "wb") as fp_:
            pickle.dump(metadata, fp_)
        # Patches the _get_containers_cache_filename module so that it returns the name of the new tempfile that
        # represents the cache file
        with patch.object(
            azureblob,
            "_get_containers_cache_filename",
            MagicMock(return_value=str(cache_file.name)),
        ):
            # Patches the _read_containers_cache_file module so that it returns what it normally would if the new
            # tempfile representing the cache file was passed to it
            plugged = azureblob._read_containers_cache_file(str(cache_file))
            with patch.object(
                azureblob,
                "_read_containers_cache_file",
                MagicMock(return_value=plugged),
            ):
                ret = azureblob._init(
                    "", container, multiple_env, environment, blob_cache_expire
                )
        fp_.close()
        os.remove(str(fp_.name))
        cache_file.close()
        self.assertEqual(ret, metadata)

    def test__get_cache_dir(self):
        """
        Tests the result of _get_cache_dir.
        """
        ret = azureblob._get_cache_dir()
        self.assertEqual(ret, "/var/cache/salt/master/pillar_azureblob")

    def test__get_cached_file_name(self):
        """
        Tests the result of _get_cached_file_name.
        """
        container = "test"
        saltenv = "base"
        path = "base/secret.sls"
        ret = azureblob._get_cached_file_name(container, saltenv, path)
        self.assertEqual(
            ret, "/var/cache/salt/master/pillar_azureblob/base/test/base/secret.sls"
        )

    def test__get_containers_cache_filename(self):
        """
        Tests the result of _get_containers_cache_filename.
        """
        container = "test"
        ret = azureblob._get_containers_cache_filename(container)
        self.assertEqual(
            ret, "/var/cache/salt/master/pillar_azureblob/test-files.cache"
        )

    def test__refresh_containers_cache_file(self):
        """
        Tests the result of _refresh_containers_cache_file to ensure that it successfully copies blob data into a
            cache file.
        """
        blob_client = MockBlobServiceClient()
        container = "test"
        cache_file = tempfile.NamedTemporaryFile()
        with patch.object(
            BlobServiceClient,
            "from_connection_string",
            MagicMock(return_value=blob_client),
        ):
            ret = azureblob._refresh_containers_cache_file(
                "", container, cache_file.name
            )
        cache_file.close()
        self.assertEqual(
            ret,
            {
                "base": {
                    "test": [
                        {
                            "container": None,
                            "name": "test.sls",
                            "prefix": None,
                            "delimiter": "/",
                            "results_per_page": None,
                            "location_mode": None,
                        }
                    ]
                }
            },
        )

    def test__read_containers_cache_file(self):
        """
        Tests the result of _read_containers_cache_file to make sure that it successfully loads in pickled metadata.
        """
        metadata = {
            "base": {
                "test": [
                    {"name": "base/secret.sls", "relevant": "include.sls"},
                    {"name": "blobtest.sls", "irrelevant": "ignore.sls"},
                ]
            }
        }
        cache_file = tempfile.NamedTemporaryFile()
        # Pickles the metadata and stores it in cache_file
        with salt.utils.files.fopen(str(cache_file), "wb") as fp_:
            pickle.dump(metadata, fp_)
        # Checks to see if _read_containers_cache_file can successfully read the pickled metadata from the cache file
        ret = azureblob._read_containers_cache_file(str(cache_file))
        fp_.close()
        os.remove(str(fp_.name))
        cache_file.close()
        self.assertEqual(ret, metadata)

    def test__find_files(self):
        """
        Tests the result of _find_files. Ensures it only finds files and not directories. Ensures it also ignore
            irrelevant files.
        """
        metadata = {
            "test": [
                {"name": "base/secret.sls"},
                {"name": "blobtest.sls", "irrelevant": "ignore.sls"},
                {"name": "base/"},
            ]
        }
        ret = azureblob._find_files(metadata)
        self.assertEqual(ret, {"test": ["base/secret.sls", "blobtest.sls"]})

    def test__find_file_meta1(self):
        """
        Tests the result of _find_file_meta when the metadata contains a blob with the specified path and a blob
            without the specified path.
        """
        metadata = {
            "base": {
                "test": [
                    {"name": "base/secret.sls", "relevant": "include.sls"},
                    {"name": "blobtest.sls", "irrelevant": "ignore.sls"},
                ]
            }
        }
        container = "test"
        saltenv = "base"
        path = "base/secret.sls"
        ret = azureblob._find_file_meta(metadata, container, saltenv, path)
        self.assertEqual(ret, {"name": "base/secret.sls", "relevant": "include.sls"})

    def test__find_file_meta2(self):
        """
        Tests the result of _find_file_meta when the saltenv in metadata does not match the specified saltenv.
        """
        metadata = {"wrong": {"test": [{"name": "base/secret.sls"}]}}
        container = "test"
        saltenv = "base"
        path = "base/secret.sls"
        ret = azureblob._find_file_meta(metadata, container, saltenv, path)
        self.assertEqual(ret, None)

    def test__find_file_meta3(self):
        """
        Tests the result of _find_file_meta when the container in metadata does not match the specified metadata.
        """
        metadata = {"base": {"wrong": [{"name": "base/secret.sls"}]}}
        container = "test"
        saltenv = "base"
        path = "base/secret.sls"
        ret = azureblob._find_file_meta(metadata, container, saltenv, path)
        self.assertEqual(ret, None)



# -*- coding: utf-8 -*-
"""
Tests for runner_returns
"""
from __future__ import absolute_import, print_function, unicode_literals

import errno
import os
import socket
import tempfile

import pytest
import salt.payload
import salt.utils.args
import salt.utils.files
import salt.utils.jid
import salt.utils.yaml
from tests.support.case import ShellCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS


@pytest.mark.windows_whitelisted
class RunnerReturnsTest(ShellCase):
    """
    Test the "runner_returns" feature
    """

    def setUp(self):
        """
        Create the temp file and master.d directory
        """
        self.job_dir = os.path.join(self.master_opts["cachedir"], "jobs")
        self.hash_type = self.master_opts["hash_type"]
        self.master_d_dir = os.path.join(self.config_dir, "master.d")
        try:
            os.makedirs(self.master_d_dir)
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise

        self.conf = tempfile.NamedTemporaryFile(
            mode="w", suffix=".conf", dir=self.master_d_dir, delete=False,
        )

    def tearDown(self):
        """
        Close the tempfile.NamedTemporaryFile object, cleaning it up
        """
        try:
            self.conf.close()
        except OSError:
            pass
        salt.utils.files.rm_rf(self.master_d_dir)
        # Force a reload of the configuration now that our temp config file has
        # been removed.
        self.run_run_plus("test.arg")

    @staticmethod
    def clean_return(data):
        """
        Remove kwargs and timestamp (things that are variable) so we have a
        stable value to assert
        """
        # Remove pub_kwargs
        data["fun_args"][1] = salt.utils.args.clean_kwargs(**data["fun_args"][1])
        data["return"]["kwargs"] = salt.utils.args.clean_kwargs(
            **data["return"]["kwargs"]
        )

        # Pop off dynamic keys in the return schema that are impossible to test.
        # Do not supply the default arguments because we want to know if we are
        # missing some aspect of the schema.
        data.pop("_stamp")

    def write_conf(self, data):
        """
        Dump the config dict to the conf file
        """
        self.conf.write(salt.utils.yaml.safe_dump(data, default_flow_style=False))
        self.conf.flush()
        self.conf.close()

    @slowTest
    def test_runner_returns_disabled(self):
        """
        Test with runner_returns disabled
        """
        self.write_conf({"runner_returns": False})
        ret = self.run_run_plus("test.arg", "foo", bar="hello world!")

        jid = ret.get("jid")
        if jid is None:
            raise Exception("jid missing from run_run_plus output")

        serialized_return = os.path.join(
            salt.utils.jid.jid_dir(jid, self.job_dir, self.hash_type),
            "master",
            "return.p",
        )
        self.assertFalse(os.path.isfile(serialized_return))

    @slowTest
    def test_runner_returns_enabled(self):
        """
        Test with runner_returns enabled
        """
        self.write_conf({"runner_returns": True})
        ret = self.run_run_plus("test.arg", "foo", bar="hello world!")

        jid = ret.get("jid")
        if jid is None:
            raise Exception("jid missing from run_run_plus output")

        serialized_return = os.path.join(
            salt.utils.jid.jid_dir(jid, self.job_dir, self.hash_type),
            "master",
            "return.p",
        )
        serial = salt.payload.Serial(self.master_opts)
        with salt.utils.files.fopen(serialized_return, "rb") as fp_:
            deserialized = serial.loads(fp_.read(), encoding="utf-8")

        self.clean_return(deserialized["return"])

        # Now we have something sane we can reliably compare in an assert.
        if "SUDO_USER" in os.environ:
            user = "sudo_{0}".format(os.environ["SUDO_USER"])
        else:
            user = RUNTIME_VARS.RUNNING_TESTS_USER
        if salt.utils.platform.is_windows():
            user = "sudo_{0}\\{1}".format(socket.gethostname(), user)
        self.assertEqual(
            deserialized,
            {
                "return": {
                    "fun": "runner.test.arg",
                    "fun_args": ["foo", {"bar": "hello world!"}],
                    "jid": jid,
                    "return": {"args": ["foo"], "kwargs": {"bar": "hello world!"}},
                    "success": True,
                    "user": user,
                }
            },
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import

# Import Salt Libs
import salt.states.tomcat as tomcat
from salt.modules import tomcat as tomcatmod

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class TomcatTestCase(TestCase, LoaderModuleMockMixin):
    """
    Validate the tomcat state
    """

    def setup_loader_modules(self):
        return {tomcat: {"__env__": "base"}}

    def test_war_deployed(self):
        """
        Test to enforce that the WAR will be deployed and
        started in the context path it will make use of WAR versions
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock_start = MagicMock(return_value="saltstack")
        mock_undeploy = MagicMock(side_effect=["FAIL", "saltstack"])
        mock_deploy = MagicMock(return_value="deploy")
        mock_ls = MagicMock(
            side_effect=[
                {"salt": {"version": "jenkins-1.20.4", "mode": "running"}},
                {"salt": {"version": "1"}},
                {"salt": {"version": "jenkins-1.2.4", "mode": "run"}},
                {"salt": {"version": "1"}},
                {"salt": {"version": "1"}},
                {"salt": {"version": "1"}},
            ]
        )
        with patch.dict(
            tomcat.__salt__,
            {
                "tomcat.ls": mock_ls,
                "tomcat.extract_war_version": tomcatmod.extract_war_version,
                "tomcat.start": mock_start,
                "tomcat.undeploy": mock_undeploy,
                "tomcat.deploy_war": mock_deploy,
            },
        ):
            ret.update({"comment": "salt with version 1.20.4" " is already deployed"})
            self.assertDictEqual(
                tomcat.war_deployed("salt", "salt://jenkins" "-1.20.4.war"), ret
            )

            with patch.dict(tomcat.__opts__, {"test": True}):
                ret.update(
                    {
                        "changes": {
                            "deploy": "will deploy salt" " with version 1.2.4",
                            "undeploy": "undeployed salt" " with version 1",
                        },
                        "result": None,
                        "comment": "",
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins" "-1.2.4.war"), ret
                )

            with patch.dict(tomcat.__opts__, {"test": False}):
                ret.update(
                    {
                        "changes": {"start": "starting salt"},
                        "comment": "saltstack",
                        "result": False,
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins" "-1.2.4.war"), ret
                )

                ret.update(
                    {
                        "changes": {
                            "deploy": "will deploy salt with" " version 1.2.4",
                            "undeploy": "undeployed salt with" " version 1",
                        },
                        "comment": "FAIL",
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins" "-1.2.4.war"), ret
                )

                ret.update(
                    {
                        "changes": {"undeploy": "undeployed salt" " with version 1"},
                        "comment": "deploy",
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins" "-1.2.4.war"), ret
                )

    def test_war_deployed_no_version(self):
        """
        Tests that going from versions to no versions and back work, as well
        as not overwriting a WAR without version with another without version.
        """
        ret = {"name": "salt", "changes": {}, "result": None, "comment": ""}

        mock_deploy = MagicMock(return_value="deploy")
        mock_undeploy = MagicMock(return_value="SUCCESS")
        mock_ls_version = MagicMock(
            return_value={"salt": {"version": "1.2.4", "mode": "running"}}
        )
        mock_ls_no_version = MagicMock(
            return_value={"salt": {"version": "", "mode": "running"}}
        )

        # We're just checking what it *would* do.
        with patch.dict(tomcat.__opts__, {"test": True}):
            with patch.dict(
                tomcat.__salt__,
                {
                    "tomcat.ls": mock_ls_version,
                    "tomcat.extract_war_version": tomcatmod.extract_war_version,
                    "tomcat.deploy_war": mock_deploy,
                    "tomcat.undeploy": mock_undeploy,
                },
            ):
                # We deploy from version to no version
                ret.update(
                    {
                        "changes": {
                            "deploy": "will deploy salt with no version",
                            "undeploy": "undeployed salt with version 1.2.4",
                        },
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins.war"), ret
                )

            with patch.dict(
                tomcat.__salt__,
                {
                    "tomcat.ls": mock_ls_no_version,
                    "tomcat.extract_war_version": tomcatmod.extract_war_version,
                    "tomcat.deploy_war": mock_deploy,
                    "tomcat.undeploy": mock_undeploy,
                },
            ):
                # Deploy from none to specified version
                ret.update(
                    {
                        "changes": {
                            "deploy": "will deploy salt with version 1.2.4",
                            "undeploy": "undeployed salt with no version",
                        },
                    }
                )
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins.war", version="1.2.4"),
                    ret,
                )
                # Deploy from none to extracted version
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins-1.2.4.war"), ret
                )
                # Don't deploy from no version to no version
                ret.update(
                    {
                        "changes": {},
                        "comment": "salt with no version is already deployed",
                        "result": True,
                    }
                )
                # Don't deploy from blank to blank version
                self.assertDictEqual(
                    tomcat.war_deployed("salt", "salt://jenkins.war"), ret
                )
                # Don't deploy from blank to false version
                self.assertDictEqual(
                    tomcat.war_deployed(
                        "salt", "salt://jenkins-1.2.4.war", version=False
                    ),
                    ret,
                )

    def test_wait(self):
        """
        Test to wait for the tomcat manager to load
        """
        ret = {
            "name": "salt",
            "changes": {},
            "result": True,
            "comment": "tomcat manager is ready",
        }
        mock = MagicMock(return_value=True)
        with patch.dict(
            tomcat.__salt__,
            {
                "tomcat.status": mock,
                "tomcat.extract_war_version": tomcatmod.extract_war_version,
            },
        ):
            self.assertDictEqual(tomcat.wait("salt"), ret)

    def test_mod_watch(self):
        """
        Test to the tomcat watcher function.
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": "True"}
        mock = MagicMock(return_value="True")
        with patch.dict(
            tomcat.__salt__,
            {
                "tomcat.reload": mock,
                "tomcat.extract_war_version": tomcatmod.extract_war_version,
            },
        ):
            ret.update({"changes": {"salt": False}})
            self.assertDictEqual(tomcat.mod_watch("salt"), ret)

    def test_undeployed(self):
        """
        Test to enforce that the WAR will be un-deployed from the server
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": "True"}
        mock = MagicMock(side_effect=[False, True, True, True, True])
        mock1 = MagicMock(
            side_effect=[
                {"salt": {"a": 1}},
                {"salt": {"version": 1}},
                {"salt": {"version": 1}},
                {"salt": {"version": 1}},
            ]
        )
        mock2 = MagicMock(side_effect=["FAIL", "saltstack"])
        with patch.dict(
            tomcat.__salt__,
            {
                "tomcat.status": mock,
                "tomcat.extract_war_version": tomcatmod.extract_war_version,
                "tomcat.ls": mock1,
                "tomcat.undeploy": mock2,
            },
        ):
            ret.update({"comment": "Tomcat Manager does not respond"})
            self.assertDictEqual(tomcat.undeployed("salt"), ret)

            ret.update({"comment": "", "result": True})
            self.assertDictEqual(tomcat.undeployed("salt"), ret)

            with patch.dict(tomcat.__opts__, {"test": True}):
                ret.update({"changes": {"undeploy": 1}, "result": None})
                self.assertDictEqual(tomcat.undeployed("salt"), ret)

            with patch.dict(tomcat.__opts__, {"test": False}):
                ret.update(
                    {"changes": {"undeploy": 1}, "comment": "FAIL", "result": False}
                )
                self.assertDictEqual(tomcat.undeployed("salt"), ret)

                ret.update({"changes": {"undeploy": 1}, "comment": "", "result": True})
                self.assertDictEqual(tomcat.undeployed("salt"), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Amel Ajdinovic <amel@stackpointcloud.com>`
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.integration.cloud.helpers.cloud_test_base import TIMEOUT, CloudTest
from tests.support.unit import skipIf

# Import Third-Party Libs
try:
    from oneandone.client import OneAndOneService  # pylint: disable=unused-import

    HAS_ONEANDONE = True
except ImportError:
    HAS_ONEANDONE = False


@skipIf(HAS_ONEANDONE is False, "salt-cloud requires >= 1and1 1.2.0")
class OneAndOneTest(CloudTest):
    """
    Integration tests for the 1and1 cloud provider
    """

    PROVIDER = "oneandone"
    REQUIRED_PROVIDER_CONFIG_ITEMS = ("api_token",)

    def test_list_images(self):
        """
        Tests the return of running the --list-images command for 1and1
        """
        image_list = self.run_cloud("--list-images {0}".format(self.PROVIDER_NAME))
        self.assertIn("coreOSimage", [i.strip() for i in image_list])

    def test_instance(self):
        """
        Test creating an instance on 1and1
        """
        # check if instance with salt installed returned
        ret_str = self.run_cloud(
            "-p oneandone-test {0}".format(self.instance_name), timeout=TIMEOUT
        )
        self.assertInstanceExists(ret_str)

        self.assertDestroyInstance()



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.layman as layman

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LaymanTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.layman
    """

    def setup_loader_modules(self):
        return {layman: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verify that the overlay is present.
        """
        name = "sunrise"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[[name], []])
        with patch.dict(layman.__salt__, {"layman.list_local": mock}):
            comt = "Overlay {0} already present".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(layman.present(name), ret)

            with patch.dict(layman.__opts__, {"test": True}):
                comt = "Overlay {0} is set to be added".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(layman.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to verify that the overlay is absent.
        """
        name = "sunrise"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[[], [name]])
        with patch.dict(layman.__salt__, {"layman.list_local": mock}):
            comt = "Overlay {0} already absent".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(layman.absent(name), ret)

            with patch.dict(layman.__opts__, {"test": True}):
                comt = "Overlay {0} is set to be deleted".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(layman.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Decorators for salt.utils.path
"""
from __future__ import absolute_import, print_function, unicode_literals

import functools

# Import Salt libs
import salt.utils.path
from salt.exceptions import CommandNotFoundError


def which(exe):
    """
    Decorator wrapper for salt.utils.path.which
    """

    def wrapper(function):
        @functools.wraps(function)
        def wrapped(*args, **kwargs):
            if salt.utils.path.which(exe) is None:
                raise CommandNotFoundError(
                    "The '{0}' binary was not found in $PATH.".format(exe)
                )
            return function(*args, **kwargs)

        return wrapped

    return wrapper


def which_bin(exes):
    """
    Decorator wrapper for salt.utils.path.which_bin
    """

    def wrapper(function):
        @functools.wraps(function)
        def wrapped(*args, **kwargs):
            if salt.utils.path.which_bin(exes) is None:
                raise CommandNotFoundError(
                    "None of provided binaries({0}) was not found "
                    "in $PATH.".format(["'{0}'".format(exe) for exe in exes])
                )
            return function(*args, **kwargs)

        return wrapped

    return wrapper



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.rabbitmq_policy as rabbitmq_policy

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class RabbitmqPolicyTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.rabbitmq_policy
    """

    def setup_loader_modules(self):
        return {rabbitmq_policy: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the RabbitMQ policy exists.
        """
        name = "HA"
        pattern = ".*"
        definition = '{"ha-mode":"all"}'

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock = MagicMock(
            side_effect=[
                {
                    "/": {
                        name: {
                            "pattern": pattern,
                            "definition": definition,
                            "priority": 0,
                        }
                    }
                },
                {},
            ]
        )
        with patch.dict(rabbitmq_policy.__salt__, {"rabbitmq.list_policies": mock}):
            comt = "Policy / HA is already present"
            ret.update({"comment": comt})
            self.assertDictEqual(
                rabbitmq_policy.present(name, pattern, definition), ret
            )

            with patch.dict(rabbitmq_policy.__opts__, {"test": True}):
                comment = "Policy / HA is set to be created"
                changes = {"new": "HA", "old": {}}
                ret.update({"comment": comment, "result": None, "changes": changes})
                self.assertDictEqual(
                    rabbitmq_policy.present(name, pattern, definition), ret
                )

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named policy is absent.
        """
        name = "HA"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(rabbitmq_policy.__salt__, {"rabbitmq.policy_exists": mock}):
            comment = "Policy '/ HA' is not present."
            ret.update({"comment": comment})
            self.assertDictEqual(rabbitmq_policy.absent(name), ret)

            with patch.dict(rabbitmq_policy.__opts__, {"test": True}):
                comment = "Policy '/ HA' will be removed."
                changes = {"new": "", "old": "HA"}
                ret.update({"comment": comment, "result": None, "changes": changes})
                self.assertDictEqual(rabbitmq_policy.absent(name), ret)
